C     FILE FOR3D.F                                VERSION 1.1 - 09/04/92
C     ******************************************************************
      PROGRAM FOR3D
C     ******************************************************************
C     *  THIS IS AN IMPROVED VERSION OF THE RESEARCH CODE FOR3D.       *
C     ******************************************************************
C     ******************************************************************
C     *  MATHEMATICAL MODEL BY:                                        *
C     *                                                                *  
C     *  DR. D. LEE , CODE 3122                                        *
C     *  ENVIRONMENTAL AND TACTICAL SUPPORT SYSTEMS DEPARTMENT         *
C     *  NAVAL UNDERWATER SYSTEMS CENTER                               *
C     *  NEW LONDON, CT 06320, U.S.A.                                  *
C     *                                                                *
C     *  PROF. M. H. SCHULTZ                                           *
C     *  DEPARTMENT OF COMPUTER SCIENCE                                *
C     *  YALE UNIVERSITY                                               *
C     *  NEW HAVEN, CT 06520, U.S.A.                                   *
C     *                                                                *
C     *  PROF. YOUCEF SAAD                                             *
C     *  DEPARTMENT OF COMPUTER SCIENCE                                *
C     *  UNIVERSITY OF MINNESOTA                                       *
C     *  MINNEAPOLIS, MN 55455-0159, U.S.A.                            *
C     *                                                                *
C     *  PROF. W. L. SIEGMANN                                          *
C     *  DEPARTMENT OF MATHEMATICAL SCIENCES                           * 
C     *  RENSSELAER POLYTECHNIC INSTITUTE                              *
C     *  TROY, NY 12180-3590, U.S.A.                                   *
C     ******************************************************************
C     *  ORIGINAL VERSION OF MATHEMATICAL MODEL REPORTED IN:           *
C     *  AN EFFICIENT METHOD FOR SOLVING THE THREE-DIMENSIONAL WIDE    *
C     *  ANGLE WAVE EQUATION, RESEARCH REPORT YALEU/DCS/RR-643,        *
C     *  YALE UNIVERSITY, NEW HAVEN, CT., OCT 1986.                    *
C     ******************************************************************
C     ******************************************************************
C     *  COMPUTER MODEL BY:                                            *
C     *  G. BOTSEAS , CODE 3122                                        *
C     *  ENVIRONMENTAL AND TACTICAL SUPPORT SYSTEMS DEPARTMENT         *
C     *  NAVAL UNDERWATER SYSTEMS CENTER                               *
C     *  NEW LONDON, CONNECTICUT 06320, U.S.A.                         *
C     ******************************************************************
C     *  ORIGINAL VERSION OF COMPUTER MODEL REPORTED IN:               *
C     *  FOR3D: A COMPUTER MODEL FOR SOLVING THE LSS THREE-DIMENSIONAL *
C     *         WIDE ANGLE WAVE EQUATION, NUSC TR 7943, 14AUG87,       *
C     *         G. BOTSEAS, D. LEE, AND D. KING                        *
C     ******************************************************************
C     ******************************************************************
C     *  PROJECT SPONSORED BY:                                         *
C     *  DNL   - PROGRAM MANAGER, DR. G.W. MORTON                      *
C     *  NUSC  - IN-HOUSE LABORATORY INDEPENDENT RESEARCH              *
C     *          PROGRAM MANAGERS, DRS. W.A. VONWINKLE AND K. LIMA     *
C     *  ONR   - PROGRAM MANAGERS, DRS. R.L. LAU, R. OBROCHTA, M. ORR, *
C     *          AND R.BAER                                            * 
C     *  NORDA - PROGRAM MANAGER, R.W. MCGIRR                          *
C     ******************************************************************
C     ******************************************************************
C     *  CRAY VERSION - FORTRAN                                        *
C     *  CRAY UNICOS JOB SCRIPT IS IN FILE FOR3D.JOB.                  *
C     ******************************************************************
C     *  MAIN PROGRAM IS FOR3D.                                        *
C     ******************************************************************
C     *  COMMON BLOCK IS IN FILE FOR3D.CMN                             *
C     *  DIMENSIONAL PARAMETERS ARE IN FILE FOR3D.PAR                  *
C     ******************************************************************  
C     *  ALPHABETICAL LIST OF SUBROUTINES FOLLOWS                      *
C     ******************************************************************
C     *  AMIFD3  - COMPUTES DIAGONALS FOR MATRIX A. CALLED BY FOR3D.   *
C     *  BCON3D  - SUPPLIES BOTTOM CONDITION. CALLED BY RHS.           *
C                - IF ITYPEB = 0, BOTX AND BOTY ARE SET TO 0.          *
C                - IF ITYPEB = 1, UBCON3D IS CALLED.                   *
C                - IF ITYPEB = 3, AN ABSORBING LAYER IS INTRODUCED.    *
C     *  BMIFD3  - COMPUTES DIAGONALS FOR MATRIX B. CALLED BY FOR3D.   *
C     *  INDX3D  - COMPUTES INDEX OF REFRACTION ARRAYS XN1 AND XN2.    *
C                - CALLED BY FOR3D.                                    *
C     *  PORT2D  - GENERATES 2D SOLUTION AT PORT SIDEWALL IF ITYPPW=2  *
C     *  PORT3D  - SUPPLIES BOUNDARY CONDITIONS AT PORT SIDEWALL.      *
C                - IF ITYPPW = 0, THE FIELD ALONG THE WALL IS SET TO 0.*
C                - IF ITYPPW = 1, UPORT3D IS CALLED.                   *
C                - IF ITYPPW = 2, PORT3D PREPARES FOR A 2D SOLUTION    *  
C                  ALONG THE PORT WALL.                                *
C                - CALLED BY RHS IF NDIM = 3.                          *
C     *  PRINTP  - PRINTS SELECTED INPUT PARAMETERS. CALLED BY FOR3D.  *
C     *  RHS     - COMPUTES RIGHT HAND SIDE OF SYSTEM OF EQUATIONS.    *
C                - CALLED BY FOR3D.                                    *
C     *  SCON3D  - SUPPLIES SURFACE CONDITION. CALLED BY RHS.          *
C                - IF ITYPES = 0, SURX AND SURY ARE SET TO 0.          *
C                - IF ITYPEB = 1, USCON3D IS CALLED.                   *
C     *  SFLD3D  - GENERATES GAUSSIAN STARTING FIELD IF ISF = 0.       *
C                - CALLS USFLD3D IF ISF = 1.                           *
C                - GENERATES GREEN'S STARTING FIELD IF ISF = 2.        *
C                - CALLED BY FOR3D.                                    *
C     *  STBD2D  - GENERATES 2D SOLUTION AT STBD SIDEWALL IF ITYPSW=2  *
C                - AND NDIM = 3. CALLED BY TWOSTEP.                    *
C     *  STBD3D  - SUPPLIES BOUNDARY CONDITIONS AT STARBOARD SIDEWALL. *
C                - IF ITYPSW = 0, THE FIELD ALONG THE WALL IS SET TO 0.*
C                - IF ITYPSW = 1, USTBD3D IS CALLED.                   *
C                - IF ITYPSW = 2, STBD3D PREPARES FOR A 2D SOLUTION.   *  
C                  ALONG THE STARBOARD WALL.                           *
C                - CALLED BY RHS IF NDIM = 3.                          *
C     *  SVP3D   - READS SOUND SPEED PROFILES IN INPUT RUNSTREAM.      *
C                - CALLED BY FOR3D IF KSVP = 0.                        *
C     *  TRID3D  - SOLVES A TRIDIAGONAL SYSTEM OF EQUATIONS.           *
C                - CALLED BY PORT2D, STBD2D, AND TWOSTEP.              *
C     *  TWOSTEP - MARCHES THE ACOUSTIC FIELD FORWARD.                 *
C                - FIRST STEP SOLVES IN RANGE.                         *
C                - SECOND STEP SOLVES IN AZIMUTH.                      *
C                - CALLED BY FOR3D.                                    *
C                - IF NDIM = 3 AND ITYPPW = 2, TWOSTEP CALLS PORT2D.   *
C                - IF NDIM = 3 AND ITYPSW = 2, TWOSTEP CALLS STBD2D.   *
C                - IF NDIM = 1 OR 2, TRID3D IS CALLED ONCE.            * 
C                - IF NDIM = 3, TRID3D IS CALLED TWICE.                * 
C     ******************************************************************  
C     *  USER SUBROUTINES                                              *
C     ******************************************************************
C     *  UBCON3D - USER SUPPLIED BOTTOM CONDITIONS. CALLED BY BCON3D.  *
C     *  UBOTTOM - USER SUPPLIED BOTTOM PROFILE INPUT ROUTINE.         *
C                - CALLED BY FOR3D IF KBOT NE 0.                       *
C     *  UEXACT  - EXACT SOLUTION. USER WRITTEN. FOR TEST PURPOSES.    *
C                - CALLED BY FOR3D.                                    *   
C     *  UPORT3D - USER SUPPLIED PORT WALL CONDITIONS. CALLED BY PORT3D*
C     *  USCON3D - USER SUPPLIED SURFACE CONDITIONS. CALLED BY SCON3D. *
C     *  USFLD3D - USER SUPPLIED STARTING FIELD.                       *
C                - CALLED BY SFLD3D IF ISF = 1.                        *
C     *  USTBD3D - USER SUPPLIED STBD WALL CONDITIONS. CALLED BY STBD3D*
C     *  USVP3D  - USER SOUND SPEED PROFILE INPUT ROUTINE.             *
C                - CALLED BY FOR3D IF KSVP NOT 0.                      *
C     ******************************************************************
C     *  ALPHABETICAL LIST OF SOME PROGRAM VARIABLES FOLLOWS:          *
C     ******************************************************************
C     *  AL    - ARRAY - LOWER DIAGONAL OF MATRIX A - COMPLEX
C     *  ALFA  - SET TO 1/12 IF DOUGLAS METHOD REQUESTED
C     *  ALPHA - VOLUME ATTENUATION - DB/METER
C     *  AM    - ARRAY - MAIN  DIAGONAL OF MATRIX A - COMPLEX
C     *  ANG   - RELATIVE BEARING - DEGREES
C     *  AU    - ARRAY - UPPER DIAGONAL OF MATRIX A - COMPLEX
C     *  ATT   - ATTENUATION COEFFICIENT FOR ARTIFICIAL ABSORBING LAYER
C     *  BETA  - ARRAY - ATTENUATION IN LAYERS - DB/WAVELENGTH
C     *  BL    - ARRAY - LOWER DIAGONAL OF MATRIX B - COMPLEX
C     *  BM    - ARRAY - MAIN DIAGONAL OF MATRIX B - COMPLEX
C     *  BOTX  - ARRAY - COMPLEX PRESSURE AT BOTTOM AT ADVANCED RANGE RA+DR
C     *  BOTY  - ARRAY - COMPLEX PRESSURE AT BOTTOM AT PRESENT RANGE RA
C     *  BTA   - ARRAY - PARTIAL SOLUTION OF SYSTEM OF EQUATIONS
C     *  BU    - ARRAY - UPPER DIAGONAL OF MATRIX B - COMPLEX
C     *  C0    - REFERENCE SPEED OF SOUND - METERS/SEC
C     *  CSVP  - ARRAY - SOUND VELOCITY - METERS/SEC
C     *  D     - ARRAY - RIGHT HAND SIDE OF SYSTEM OF EQUATIONS - COMPLEX
C     *  DEG   - CONVERSION FACTOR - DEGREES/RADIAN
C     *  DOUGRA - 0     - DON'T USE DOUGLAS METHOD
C                 NOT 0 - RANGE AT WHICH TO BEGIN DOUGLAS METHOD - METERS
C     *  DR    - RANGE STEP - METERS
C     *  DTH   - WIDTH OF SECTOR - DEGREES
C     *  DZ    - DEPTH INCREMENT OF SOLUTION - METERS
C     *  DZZ   - DEPTH INCREMENT FOR ADJUSTING LAYER DEPTHS IN SLOPING BOTTOM
C     *  FLDW  - WIDTH OF FIELD - DEGREES
C     *  FRQ   - FREQUENCY - HZ
C     *  HNK   - HANKEL FUNCTION H0(1)
C     *  HNKL  - EXTERNAL FUNCTION - COMPUTES HANKEL FUNCTION H0(1)
C     *  IBOT  - BOTTOM DEPTH PRINT FLAG
C                0 - DO NOT PRINT BOTTOM DEPTHS
C                1 - PRINT BOTTOM DEPTHS
C     *  IHNK  - HANKEL FUNCTION FLAG
C                0 - HANKEL FUNCTION NOT USED. 10*LOG(R) ADDED TO SOLUTION.
C                1 - STARTING FIELD DIVIDED BY HANKEL FUNCTION.
C                    SOLUTION MULTIPLIED BY HANKEL FUNCTION BEFORE
C                    COMPUTING PROPAGATION LOSS.
C     *  ILYR  - INDEX FOR ARRAYS BETA, ZLYR, AND RHO
C     *  IPZ   - EVERY IPZ'TH VALUE IN DEPTH IS PRINTED
C     *  ISF   - STARTING FIELD FLAG
C                0 - PROGRAM GENERATES GAUSSIAN STARTING FIELD
C                    AT RANGE = 0.0. SEE SUBROUTINE SFLD3D.
C                1 - USER SUPPLIES STARTING FIELD. SEE SUBROUTINE USFLD3D.
C                2 - GREENS WIDE ANGLE STARTER
C                3 - AVAILABLE
C     *  ISFLD - STARTING FIELD PRINT FLAG
C                0 - DO NOT PRINT STARTING FIELD
C                1 - PRINT STARTING FIELD
C     *  ISVP  - SVP PRINT FLAG
C                0 - DO NOT PRINT SOUND VELOCITY PROFILE
C                1 - PRINT SOUND VELOCITY PROFILE
C     *  ITEMP - TEMPORARY VARIABLE
C     *  ITRK  - ARRAY - INDICES FOR ARRAY TRACK
C     *  ITYPEB - TYPE OF BOTTOM
C                 0 - BCON3D SETS BOTY AND BOTX = 0.0
C                 1 - USER SUPPLIES BOTTOM CONDITION. SEE SUBROUTINE UBCON3D.
C                 2 - SPARE.
C                 3 - ABSORBING LAYER INTRODUCED - BOTTOM OF LAYER IS FLAT
C                 4 - SPARE.
C     *  ITYPES - TYPE OF SURFACE
C                 0 - PRESSURE RELEASE. SCON3D SETS SURY AND SURX = 0.0
C                 1 - USER SUPPLIES SURFACE CONDITION. SEE SUBROUTINE USCON3D.
C                 2 - SPARE
C     *  ITYPPW - TYPE OF PORT SIDEWALL BOUNDARY CONDITION.
C                 0 - FIELD ALONG PORT SIDEWALL IS SET TO 0.0.
C                 1 - USER SUPPLIED. SEE SUBROUTINE UPORT3D.
C                 2 - MODEL GENERATES 2D SOLUTION IF NDIM = 3.
C     *  ITYPSW - TYPE OF STBD SIDEWALL BOUNDARY CONDITION.
C                 0 - FIELD ALONG STBD SIDEWALL IS SET TO 0.0.
C                 1 - USER SUPPLIED. SEE SUBROUTINE USTBD3D.
C                 2 - MODEL GENERATES 2D SOLUTION IF NDIM = 3.
C     *  IWZ    - INDEX INCREMENT OF RECEIVER SOLUTIONS TO BE WRITTEN ON DISK.
C     *  IXSVP  - ARRAY OF POINTERS WHICH POINT TO ENTRIES IN CSVP AND ZSVP.
C               - IXSVP(1,L) POINTS TO BOTTOM DEPTH AND SPEED IN LAYER 1
C                 ON SECTOR BOUNDARY L.
C               - IXSVP(2,L) POINTS TO BOTTOM DEPTH AND SPEED IN LAYER 2
C                 ON SECTOR BOUNDARY L.
C                 ETC.
C     *  KSVP   - SVP PROFILE FLAG.
C                 0      - PROFILE IS IN RUNSTREAM - SEE SUBROUTINE SVP3D.
C                 NOT 0  - USER ROUTINE GENERATES PROFILES - SEE SUBROUTINE
C                          USVP3D.
C               - KSVP MAY BE USED AS AN INDEX IN A GOTO STATEMENT IN USVP3D.
C     *  KBOT   - BOTTOM PROFILE FLAG.
C                 0      - PROFILE IS IN RUNSTREAM .
C                 NOT 0  - USER ROUTINE GENERATES PROFILES - SEE SUBROUTINE
C                          UBOTTOM.
C     *  MLYR  - TEMPORARY - NUMBER OF LAYERS INVOLVED IN SPECIFIC CALCULATION.
C     *  MM    - INDEX - MM+1 POINTS TO FIRST VALUE OF ARTIFICIAL ABSORBING
C                        LAYER IN ARRAY U.
C     *  MXLYR - PARAMETER - MAXIMUM NUMBER OF LAYERS. 
C                           - MAX DIMENSION OF ARRAYS BETA,RHO,ZLYR,IXSVP.
C     *  MXN   - PARAMETER - MAXIMUM DIMENSION OF C, X, Y, AND U ARRAYS.
C     *  MXSVP - PARAMETER - MAXIMUM DIMENSION OF ARRAYS CSVP AND ZSVP.
C     *  MXTRK - PARAMETER - MAXIMUM DIMENSION OF ARRAY TRACK.
C     *  N     - NUMBER OF EQUI-SPACED POINTS IN U. 
C              - INCLUDES BOTTOM POINT - DOES NOT INCLUDE SURFACE POINT.
C     *  N1    - DIAGONAL ELEMENTS N1 THRU N WILL BE COMPUTED - NOT USED.
C     *  NA    - NUMBER OF POINTS IN ABSORBING LAYER.
C     *  NDIV  - IF DOUGLAS METHOD IS REQUESTED, DIVIDE N BY NDIV.
C     *  NIU   - PARAMETER - INPUT UNIT NUMBER.
C     *  NLYR  - NUMBER OF LAYERS.
C     *  NOLD  - NUMBER OF RECEIVER DEPTHS ON ENTRY TO ROUTINE CRNK.
C     *  NOU   - PARAMETER - OUTPUT UNIT NUMBER.
C     *  NPU   - PARAMETER - PRINTER UNIT NUMBER.
C     *  NSEC  - NUMBER OF SECTORS.
C     *  NSOL  - NUMBER OF SOLUTIONS (NSEC+1).
C     *  NSVP  - NUMBER OF POINTS IN CSVP AND ZSVP.
C     *  NTOT  - TOTAL POINTS IN SOLUTION (N*NSOL).
C     *  NWSVP - NUMBER OF POINTS IN LAYER 1 SVP.
C     *  NZ    - NUMBER OF SOLUTION DEPTHS TO BE WRITTEN ON DISK.
C     *  OLDR  - RANGE INCREMENT AT START OF PROBLEM.
C     *  PDR   - RANGE INCREMENT AT WHICH SOLUTION IS PRINTED - METERS.
C     *  PDTH  - AZIMUTHAL INCREMENT AT WHICH SOLUTION IS PRINTED - DEGREES.
C     *  PDZ   - DEPTH INCREMENT AT WHICH SOLUTION IS PRINTED - METERS.
C     *  PHI   - WIDTH OF SECTOR IN RADIANS.
C     *  PI    - 3.141592654
C     *  PL    - PROPAGATION LOSS - DB.
C     *  PORTX - ARRAY - COMPLEX FIELD AT PORT SIDEWALL - ADVANCED RANGE.
C     *  PORTY - ARRAY - COMPLEX FIELD AT PORT SIDEWALL - PRESENT RANGE.
C     *  R0    - INITIAL RANGE - METERS.
C     *  R1    -      RANGE AT WHICH BOTTOM DEPTH IS AVAILABLE - METERS.
C     *  R2    - NEXT RANGE AT WHICH BOTTOM DEPTH IS AVAILABLE - METERS.
C     *  RA    - HORIZONTAL RANGE OF STARTING FIELD FROM SOURCE - METERS.
C              - RA IS SET TO 0.0 IF STARTING FIELD IS GAUSSIAN. RA IS 
C              - INCREMENTED BY DR AS SOLUTION IS MARCHED OUT IN RANGE.
C     *  RA+DR - RANGE TO WHICH SOLUTION IS TO BE ADVANCED - METERS.
C     *  RHO   - ARRAY - DENSITY IN LAYER .
C     *  RHOG  - ARRAY - DENSITY GRADIENT IN LAYER.
C     *  RMAX  - MAXIMUM RANGE OF SOLUTION - METERS.
C     *  RSVP  - NEXT RANGE AT WHICH SVP IS AVAILABLE - METERS.
C     *  STBDX - ARRAY - COMPLEX FIELD AT STBD SIDEWALL - ADVANCED RANGE.
C     *  STBDY - ARRAY - COMPLEX FIELD AT STBD SIDEWALL - PRESENT RANGE.
C     *  SURX  - ARRAY - COMPLEX PRESSURE AT SURFACE AT ADVANCED RANGE RA+DR.
C     *  SURY  - ARRAY - COMPLEX PRESSURE AT SURFACE AT PRESENT RANGE RA.
C     *  TEMP  - TEMPORARY VARIABLE - COMPLEX.
C     *  THETA - ARRAY - SLOPE OF BOTTOM ALONG SECTOR BOUNDARIES - RADIANS.
C                .EQ.0 -- FLAT BOTTOM .
C                .GT.0 -- SHALLOW TO DEEP.
C                .LT.0 -- DEEP TO SHALLOW.
C     *  TITLE - 80 CHARACTER TITLE.
C     *  TM    - ARRAY - TIME OF DAY.
C     *  TRACK - 3 DIM. ARRAY - RANGE AND DEPTH OF WATER - METERS.
C              - ASSUMES FLAT BOTTOM AT PRESENT TIME.
C     *  U     - ARRAY - COMPLEX ACOUSTIC PRESSURE FIELD.
C     *  U1-U12 - USER MAY USE THESE VARIABLES TO INPUT DATA REQUIRED BY USER
C               - ROUTINES - REAL - IN COMMON BLOCK.
C     *  UA-UE  - USER MAY USE THESE VARIABLES - COMPLEX - IN COMMON BLOCK
C     *  V      - ARRAY - PARTIAL SOLUTION OF COMPLEX FIELD
C     *  WDR    - RANGE STEP AT WHICH SOLUTION IS WRITTEN ON DISK - METERS
C     *  WDTH   - AZIMUTHAL INCREMENT AT WHICH SOLUTION IS WRITTEN ON DISK - DEG
C     *  WDZ    - DEPTH INCREMENT AT WHICH SOLUTION IS WRITTEN ON DISK - METERS
C               - WDZ SHOULD BE SELECTED SO THAT PLOT PROGRAM DOES NOT 
C               - INTERPOLATE BETWEEN WIDELY SPACED RECEIVERS.
C     *  WZ1    - FIRST RECEIVER DEPTH AT WHICH SOLUTION IS WRITTEN ON DISK 
C     *  WZ2    - LAST RECEIVER DEPTH AT WHICH SOLUTION IS WRITTEN ON DISK
C               - IN OTHER WORDS, WRITE WZ1 TO WZ2 BY WDZ - METERS
C     *  XK0    - REFERENCE WAVE NUMBER
C     *  XN2    - ARRAY - INDEX OF REFRACTION SQUARED - COMPLEX
C     *  XPR    - RANGE AT WHICH SOLUTION IS PRINTED - METERS
C     *  XWR    - RANGE AT WHICH SOLUTION IS WRITTEN ON DISK - METERS
C     *  Z1     - DEPTH OF WATER AT RANGE R1 - METERS
C     *  Z2     - DEPTH OF WATER AT RANGE R2 - METERS
C     *  ZA     - DEPTH OF FIELD AT RANGE RA - METERS
C               - INITIAL DEPTH OF STARTING FIELD AT RANGE RA IS
C               - AS FOLLOWS:
C               - IF ITYPEB = 0 OR 1, ZA IS MAXIMUM DEPTH OF 
C               - BOTTOM-MOST SEDIMENT LAYER AT INITIAL RANGE OF
C               - STARTING FIELD. IF ITYPEB = 3, ZA IS MAXIMUM
C               - DEPTH OF ARTIFICIAL ABSORBING LAYER AT INITIAL 
C               - RANGE OF STARTING FIELD. PROGRAM INSERTS LAYER.
C               - RHO AND BETA ARE OBTAINED FROM LAYER ABOVE.
C               - SPEED IS BOTTOM-MOST SPEED FROM LAYER ABOVE.
C               - BOTTOM OF ABSORBING LAYER REMAINS FLAT.
C     *  ZI     - DEPTH OF RECEIVER 'I' - METERS.
C     *  ZLYR   - ARRAY - DEPTH TO BOTTOM OF LAYER - METERS.
C     *  ZS     - SOURCE DEPTH - METERS.
C     *  ZSVP   - ARRAY - DEPTH OF SOUND VELOCITY - METERS.
C     ******************************************************************
C     ******************************************************************
C     *  INPUT UNIT NUMBER = NIU
C     *  INPUT FILE NAME   = for3din - SEE CRAY UNICOS JOB SCRIPT
C     *  CONTENTS: INPUT RUNSTREAM IN FREE FORMAT
C
C  LINE    INPUT DATA
C
C    1     TITLE
C
C    2     NDIM
C
C    3     FRQ,ZS,C0,ISF,RA,ZA,N,IHNK,ITYPES,ITYPEB,ITYPPW,ITYPSW,FLDW,NSEC
C
C    4     RMAX,DR,WDR,WZ1,WZ2,WDZ,WDTH,PDR,PDZ,PDTH,ISFLD,ISVP,IBOT
C
C    5     DOUGRA,NDIV
C
C    6     U1,U2,U3,U4,U5,U6,U7,U8,U9,U10,U11,U12
C
C    7     KBOT
C
C    8     R1,Z1  **     IF KBOT = 0, BOTTOM PROFILE IS IN RUNSTREAM.
C    9     R2,Z2    *    RANGE, WATER DEPTH (METERS).
C            .      **   -1,-1 MARKS END OF THIS PROFILE.
C            .      ***  ENTER PROFILE FOR EACH SECTOR BOUNDARY.
C            .      **   PORT BOUNDARY FIRST.
C    N       .      *    IF KBOT NE 0, UBOTTOM IS CALLED. OMIT 
C    N+1   -1,-1  **     LINES 8 THRU N+1.
C
C             ************************************************  
C    N+2  RSVP                                                *
C    N+3  KSVP                                                *
C             **************************************          *
C    N+4  NLYRS(L)                                  *         *
C             ****************************          *         **
C    N+5  ZLYR(I,L),RHO(I,L),RHOG(I,L),   *         **        *** NOTE
C                   BETA(I,L),BETAG(I,L)  **        *** NOTE  ***  1
C    N+6  ZSVP(1,L),CSVP(1,L)             *** NOTE  ***  2    **    
C    N+7  ZSVP(2,L),CSVP(2,L)             **   3    **        *
C     .          .       .                *         *         *         
C    N+M  ZSVP(J,L),CSVP(J,L) ************          *         *        
C                             **********************          * 
C                             ********************************             
C 
C     ******************************************************************
C     *** QUICK REFERENCE AND NOTES FOR INPUT RUNSTREAM
C     *** UNITS: METERS AND METERS/SEC EXCEPT AS NOTED
C     ******************************************************************
C  LINE   INPUT DATA
C
C   1     TITLE  =  USER NOTE. 80 CHARACTERS MAXIMUM.
C
C   2     NDIM   =  NUMBER OF DIMENSIONS.
C                   1 - 2D SOLUTION.
C                   2 - N x 2D SOLUTIONS.
C                   3 - 3D SOLUTIONS.
C
C   3     FRQ    =  FREQUENCY (HZ)
C
C         ZS     =  SOURCE DEPTH
C
C         C0     =  REFERENCE SOUND SPEED. IF C0 = 0.0, C0 IS SET TO AVERAGE 
C                   SPEED IN FIRST LAYER.
C
C         ISF    =  STARTING FIELD FLAG.
C                   0 = GAUSSIAN.
C                   1 = USER FIELD.
C                   2 = GREENS STARTER.
C
C         RA     =  HORIZONTAL RANGE FROM SOURCE TO STARTING FIELD.
C                   RA IS SET TO 0.0 IF ISF = 0.
C
C         ZA     =  DEPTH OF STARTING FIELD AT RANGE RA. IF ZA = 0.0, ZA IS SET
C                   TO MAX DEPTH OF BOTTOM LAYER IN FIRST PROFILE. IF ITYPEB = 
C                   2 OR 3 AND ZA = 0.0, ZA IS SET TO (4/3)*MAX DEPTH OF BOTTOM
C                   LAYER. IF ITYPEB = 3 AND ZA NOT ZERO, THE ARTIFICIAL
C                   BOTTOM LAYER IS EXTENDED TO ZA METERS PROVIDED THAT ZA
C                   IS GREATER THAN OR EQUAL TO MAX DEPTH OF BOTTOM LAYER
C                   IN FIRST PROFILE.
C
C         N      =  NUMBER OF EQUISPACED RECEIVERS IN STARTING FIELD. IF N = 0,
C                   N IS SET SO THAT THE RECEIVER DEPTH INCREMENT IS
C                   EQUAL TO 1 METER. IF N IS GREATER THAN MXN 
C                   N IS SET TO MXN.
C
C         IHNK   =  HANKEL FUNCTION FLAG. IHNK = 0, DON'T USE HANKEL FUNCTION.
C                   IHNK = 1, DIVIDE STARTING FIELD BY HANKEL FUNCTION, THEN
C                   MULTIPLY THE SOLUTION FIELD BY HANKEL FUNCTION BEFORE 
C                   COMPUTING PROPAGATION LOSS. IF STARTING FIELD IS GAUSSIAN,
C                   IHNK SHOULD BE SET TO 0. IF STARTING FIELD IS ELLIPTIC,
C                   IHNK SHOULD BE SET TO 1.
C
C         ITYPES =  TYPE OF SURFACE
C                   0 - PRESSURE RELEASE. SCON3D SETS SURY AND SURX = 0.0
C                   1 - USER SUPPLIES SURFACE CONDITION. SEE SUBROUTINE USCON3D.
C                   2 - SPARE.
C
C         ITYPEB =  TYPE OF BOTTOM
C                   0 - PRESSURE RELEASE. BCON3D SETS BOTY AND BOTX = 0.0
C                   1 - USER SUPPLIES BOTTOM CONDITION. SEE SUBROUTINE UBCON3D.
C                   2 - SPARE.
C                   3 - ABSORBING LAYER INTRODUCED - FLAT BOTTOM
C                   4 - SPARE
C
C         ITYPPW =  TYPE OF PORT SIDEWALL BOUNDARY CONDITION
C                   0 - FIELD ALONG PORT SIDEWALL IS SET TO 0.0
C                   1 - USER SUPPLIED. SEE SUBROUTINE UPORT3D.
C                   2 - MODEL GENERATES 2D SOLUTION IF NDIM = 3.
C
C         ITYPSW =  TYPE OF STARBOARD SIDEWALL BOUNDARY CONDITION
C                   0 - FIELD ALONG STARBOARD SIDEWALL IS SET TO 0.0
C                   1 - USER SUPPLIED. SEE SUBROUTINE USTBD3D.
C                   2 - MODEL GENERATES 2D SOLUTION IF NDIM = 3.
C
C         FLDW   =  WIDTH OF FIELD IN DEGREES. IGNORED IF NDIM = 1.
C
C         NSEC   =  NUMBER OF SECTORS IN FIELD. IGNORED IF NDIM = 1.
C                   NUMBER OF SOLUTIONS = NSOL = NSEC+1.  
C  
C  4      RMAX   =  MAXIMUM RANGE OF SOLUTION
C
C         DR     =  RANGE STEP. IF DR = 0, DR IS SET TO 1 METER.
C                   IF BOTTOM OF PROBLEM IS NOT FLAT, DR IS RECOMPUTED
C                   SO THAT MAX DEPTH IS EITHER INCREMENTED OR DECREMENTED
C                   BY DZ. SOLUTION IS COMPUTED EVERY DR METERS.
C
C         WDR    =  RANGE STEP AT WHICH SOLUTION IS WRITTEN ON DISK.
C                   IF WDR NOT 0, AN OUTPUT DISK FILE IS ASSIGNED. 
C                   WDR IS ROUNDED TO NEAREST DR.
C
C         WZ1    =  FIRST RECEIVER DEPTH AT WHICH SOLUTION IS WRITTEN ON DISK. 
C
C         WZ2    =  LAST RECEIVER DEPTH AT WHICH SOLUTION IS WRITTEN ON DISK.
C
C         WDZ    =  DEPTH INCREMENT AT WHICH SOLUTION IS WRITTEN ON DISK.
C                   ROUNDED TO NEAREST DZ.
C
C         WDTH   =  AZIMUTHAL INCREMENT AT WHICH SOLUTION IS WRITTEN ON DISK.
C                   ROUNDED TO NEAREST DTH.
C
C         PDR    =  RANGE STEP AT WHICH SOLUTION IS PRINTED.
C                   ROUNDED TO NEAREST DR.
C
C         PDZ    =  DEPTH INCREMENT AT WHICH SOLUTION IS PRINTED.
C                   ROUNDED TO NEAREST DZ.
C
C         PDTH   =  AZIMUTHAL INCREMENT AT WHICH SOLUTION IS PRINTED.
C                   ROUNDED TO NEAREST DTH.
C
C         ISFLD  =  0 - DON'T PRINT STARTING FIELD.
C                =  1 - PRINT STARTING FIELD.
C
C         ISVP   =  0 - DON'T PRINT SOUND VELOCITY PROFILE.
C                =  1 - PRINT SOUND VELOCITY PROFILE.
C
C         IBOT   =  0 - DON'T PRINT BOTTOM DEPTHS.
C                =  1 - PRINT BOTTOM DEPTHS.
C  
C  5      DOUGRA =  RANGE AT WHICH TO SWITCH FROM CRANK-NICOLSON METHOD TO
C                   DOUGLAS METHOD. IF DOUGRA = 0, USE CRANK-NICOLSON METHOD 
C                   ONLY. SUGGESTED VALUE IS 5000 METERS.
C
C         NDIV   =  IF DOUGRA NE 0, N IS DIVIDED BY NDIV RESULTING IN ONLY
C                   N/NDIV SOLUTIONS IN DEPTH. SPEED UP IS NDIV TIMES.
C                   SUGGESTED VALUE IS 5.
C                   
C  6      U1-U12 =  USER VARIABLES - REAL, SINGLE PRECISION.
C                   SEE HARVARD SUBROUTINE HARVARD.F FOR EXAMPLE.
C
C  7      KBOT   =  BOTTOM PROFILE FLAG.  
C                =  0 - BOTTOM PROFILE IN INPUT RUNSTREAM.
C                =  NOT ZERO. PROFILE IS SUPPLIED BY SUBROUTINE UBOTTOM.
C                   USER WRITES UBOTTOM. IF NSOL IS LARGE, UBOTTOM IS
C                   PREFERRED METHOD OF INPUT.
C
C                  BOTTOM PROFILE AT LEFTMOST SECTOR BOUNDARY.
C  8      R1,Z1  = RANGE AND DEPTH OF WATER.
C  9      R2,Z2  = ETC. 
C           .
C  N+1    -1,-1  = MARKS THE END OF THIS BOTTOM PROFILE.
C
C                  NEXT PROFILE.
C  8      R1,Z1  = RANGE AND DEPTH OF WATER.
C  9      R2,Z2  = ETC. 
C           .
C  N+1    -1,-1  = MARKS THE END OF THIS BOTTOM PROFILE.
C
C                  ETC. REPEAT UNTIL ALL PROFILES ENTERED.
C  
C  N+2    RSVP   = RANGE OF SOUND SPEED PROFILES.
C                  SEE NOTE 1.
C  
C  N+3    KSVP   = SVP FLAG.
C                = 0 - SOUND SPEED PROFILES IN INPUT RUNSTREAM.
C                = NOT ZERO.  PROFILE (LINES N+4 THRU N+M) IS SUPPLIED BY
C                  USER. USER WRITES SUBROUTINE USVP3D. KSVP MAY BE USED IN
C                  COMPUTED GOTO STATEMENT TO TRANSFER CONTROL IN USVP3D.
C                  IF NSOL IS LARGE, USVP3D IS PREFERRED METHOD OF INPUT.
C     
C  N+4    NLYRS(L) = NUMBER OF LAYERS. IF ITYPEB  = 3, PROGRAM INSERTS
C                    AN ARTIFICIAL LAYER AND INCREMENTS NLYRS(L) BY 1.
C                    SEE NOTE 2.
C  
C  N+5    ZLYR(I,L) = MAX DEPTH OF LAYER I IN PROFILE.
C
C         RHO(I,L)  = DENSITY IN LAYER I (G/CM**3).
C
C         RHOG(I,L) = DENSITY GRADIENT IN LAYER I (G/CM**3/M).
C
C         BETA(I,L) = ATTENUATION IN LAYER I (DB/WAVELENGTH).
C                     IF BETA(I,L) IS NEGATIVE, ATTENUATION IS COMPUTED.
C                     SEE NOTE 3.
C 
C         BETAG(I,L)= ATTENUATION GRADIENT IN LAYER I (DB/WAVELENGTH/M).
C  
C  N+6    ZSVP(1,L) = DEPTH TO TOP OF LAYER I
C
C         CSVP(1,L) = SPEED OF SOUND AT TOP OF LAYER I
C            .
C            .
C  N+M    ZSVP(J,L) = DEPTH TO BOTTOM OF LAYER I
C
C         CSVP(J,L) = SPEED OF SOUND AT BOTTOM OF LAYER I
C                     IF ONLY ONE SVP INPUTTED, IT IS USED THRU ENTIRE PROBLEM.
C                     IF MORE THAN ONE SVP INPUTTED, LAST SVP IS USED THRU
C                     REMAINDER OF PROBLEM.
C
C  NOTE 1  REPEAT LINES N+2 THRU N+M FOR EACH SET OF PROFILES TO BE ENTERED.
C          A SET OF PROFILES CONSISTS OF NSOL (NSEC+1) PROFILES LOCATED ON 
C          SECTOR BOUNDARIES AND EQUISPACED FROM PORT TO STARBOARD IN AZIMUTH.
C          NSEC IS THE NUMBER OF SECTORS IN THE FIELD OF INTEREST. NSEC 
C          ADJACENT SECTORS HAVE NSOL BOUNDARIES.
C 
C  NOTE 2  REPEAT LINES N+4 THRU N+M FOR EACH PROFILE IN THE SET. AT RANGE ZERO
C          ALL PROFILES ARE THE SAME. BEYOND RANGE ZERO, L PROFILES ARE
C          ENTERED FROM PORT TO STARBOARD AS VIEWED FROM RANGE R0. L IS
C          VARIED FROM 1 THRU NSOL. ENTERING -1,-1 AFTER PROFILE L 
C          WILL CAUSE THAT PROFILE TO BE ENTERED REPEATEDLY UNTIL
C          NSOL PROFILES HAVE BEEN ENTERED.
C
C NOTE 3  REPEAT LINES N+5 THRU N+M FOR EACH LAYER I IN THE PROFILE.
C         J IS THE NUMBER OF POINTS IN LAYER I.
C 
C     ******************************************************************
C     *** OUTPUT
C     ******************************************************************
C         OUTPUT UNIT NUMBER = NOU
C         OUTPUT FILE NAME   = for3dout - SEE CRAY UNICOS JOB SCRIPT 
C         CONTENTS:  AS FOLLOWS - UNFORMATTED
C 
C         THE FOLLOWING VARIABLES ARE WRITTEN ONCE:
C
C   WRITE(NOU)NDIM,FRQ,ZS,C0,ISF,RA,ZA,N,IHNK,ITYPES,ITYPEB,ITYPPW,ITYPSW,FLDW,
C             NSEC,NSOL,RMAX,DR,WDR,WZ1,WZ2,WDZ,DZ,DOUGRA,NDIV,
C             U1,U2,U3,U4,U5,U6,U7,U8,U9,U10,U11,U12
C 
C    THE VALUES OF THE VARIABLES AND ARRAYS LISTED IN THE FOLLOWING
C    WRITE STATEMENT ARE WRITTEN AT R0 AND AT EACH WDR (WRITE RANGE INCREMENT)
C    THEREAFTER.
C
C   WRITE(NOU)ANG,NZ,RA,WZ1,WDZ,(U(M+I),I=IWZ1,IWZ2,IWZ) 
C    
C   where  M   = (J-1)*N
C          J   = SOLUTION INDEX. J VARIES FROM 1 TO NSOL. PORT TO STARBOARD.
C          N   = NUMBER OF RECEIVERS IN VERTICAL COLUMN.
C          ANG = RELATIVE BEARING OF SOLUTION IN DEGREES. PORT IS NEGATIVE.
C          NZ  = NUMBER OF RECEIVERS WRITTEN ON DISK.
C          RA  = RANGE IN METERS.
C          WZ1 = DEPTH OF FIRST RECEIVER.
C          WDZ = DEPTH INCREMENT OF RECEIVERS WRITTEN ON DISK IN METERS.
C          U   = SOLUTION ARRAY. COMPLEX.
C
C     PRINTED OUTPUT IS DIRECTED TO PRINTER UNIT NUMBER NPU.
C     ******************************************************************
C     ******************************************************************
C     ******************************************************************
C     
C     *** INCLUDE COMMON BLOCK 
      INCLUDE 'FOR3D.CMN'
C
C     *** TIME OF DAY AT START OF RUN - CRAY ROUTINE
CCC      CALL SECOND(CPT1)
C
C     *** READ INPUT PARAMETERS. 
C     *** FILES ARE LINKED TO UNIT NUMBERS IN CRAY JOB SCRIPT.
C      OPEN(NIU)
	CALL ASSIGN(NIU,'FOR3D.IN')
C
      READ(NIU,15)TITLE
15    FORMAT(80A1)
C
C     *** READ SOLUTION DIMENSION
      READ(NIU,*)NDIM
      IF(NDIM.LT.1.OR.NDIM.GT.3)THEN
        WRITE(NPU,*)'INPUT ERROR. CHECK NDIM VALUE.'
        STOP
      ELSE
      ENDIF
C 
      READ(NIU,*)FRQ,ZS,C0,ISF,RA,ZA,N,IHNK,ITYPES,ITYPEB,
     CITYPPW,ITYPSW,FLDW,NSEC
C
      READ(NIU,*) RMAX,DR,WDR,WZ1,WZ2,WDZ,WDTH,PDR,PDZ,PDTH,
     CISFLD,ISVP,IBOT
C
      READ(NIU,*) DOUGRA,NDIV
C
      READ(NIU,*) U1,U2,U3,U4,U5,U6,U7,U8,U9,U10,U11,U12
C
C     SET DOUGLAS COEFFICIENT TO 0
      ALFA=0.0
C
C     *** IF C0 NE 0.0 COMPUTE REFERENCE WAVE NUMBER
      IF(C0.NE.0.0)XK0=2.0*PI*FRQ/C0
C
C     *** COMPUTE NUMBER OF SOLUTIONS. ONE FOR EACH SECTOR BOUNDARY.
C
      IF(NDIM.EQ.1)THEN
C       *** USER REQUESTED A 2D SOLUTION. ONLY ONE SOLUTION IS GENERATED.
        FLDW=0.0
        NSEC=0
        NSOL=1
        NSOLW=1
        NSOLP=1
        DTH=0
        WDTH=0.0
        PDTH=0.0
        ANG=0.0
        PHI=0.0
C
      ELSE
C       *** MORE THAN 1 SOLUTION.
        NSOL=NSEC+1
C       *** COMPUTE SECTOR WIDTH IN DEGREES. 
        DTH=FLDW/NSEC
C       *** COMPUTE SECTOR WIDTH IN RADIANS. 
        PHI=PI*FLDW/(NSEC*180)
C       *** COMPUTE NUMBER OF SOLUTIONS IN AZIMUTH TO WRITE TO DISK.
C       *** AVOID CRAY ROUND UP PROBLEMS
        RNSOLW=(WDTH/DTH+.5)
        NSOLW=INT(RNSOLW)
        IF(NSOLW.EQ.0)NSOLW=1
C       *** COMPUTE NUMBER OF SOLUTIONS IN AZIMUTH TO PRINT.
        RNSOLP=(PDTH/DTH+.5)
        NSOLP=INT(RNSOLP)
        IF(NSOLP.EQ.0)NSOLP=1
      ENDIF
C
      IF(NSOL.GT.MXSOL) THEN
        WRITE(NPU,*)'TOO MANY SOLUTIONS REQUESTED.'
        WRITE(NPU,*)'INCREASE PARAMETER MXSEC AND RECOMPILE.'
        STOP
      ELSE
      ENDIF
C
C     *** IF GAUSSIAN STARTING FIELD, RA MUST BE 0.
      IF(ISF.EQ.0) RA=0.0
      R0=RA
C
C     *** READ BOTTOM PROFILE FLAG
      READ(NIU,*) KBOT
C
C     *** IS PROFILE IN RUNSTREAM?
      IF(KBOT.EQ.0) THEN
C       *** YES
C       *** READ BOTTOM PROFILE - RANGE,DEPTH
        ZBOT=0.0
        DO 24 L=1,NSOL
        DO 22 I=1,MXTRK+1
        READ(NIU,*) TRACK(I,1,L),TRACK(I,2,L)
C       *** END OF PROFILE?
        IF(TRACK(I,1,L).LT.0.0) GO TO 23
C       *** NO
        NTRK=I
        IF(ZBOT.LT.TRACK(I,2,L))ZBOT=TRACK(I,2,L)
22      CONTINUE
23      CONTINUE
C
C       *** ERROR DETECTED?
          IF(NTRK.EQ.1.OR.NTRK.GT.MXTRK) THEN
C           *** YES.
            ITEMP=MXTRK
            WRITE(NPU,29) ITEMP
            STOP
          ELSE
C           *** NO ERROR DETECTED.
C           *** EXTEND LAST DEPTH BEYOND MAX RANGE.
            TRACK(NTRK+1,1,L)=1.0E+38
            TRACK(NTRK+1,2,L)=TRACK(NTRK,2,L)
            R2(L)=TRACK(1,1,L)
            Z2(L)=TRACK(1,2,L)
          ENDIF
24      CONTINUE
C
      ELSE
C       *** USER WRITTEN SUBROUTINE UBOTTOM SUPPLIES PROFILE.
        CALL UBOTTOM
      ENDIF
C
C     *** FIND BOTTOM SEGMENT WHICH CONTAINS STARTING RANGE
      DO 31 L=1,NSOL
      ITRK(L)=1
25    R1(L)=R2(L)
      Z1(L)=Z2(L)
      ITRK(L)=ITRK(L)+1
C
      IF(ITRK(L).GT.MXTRK) THEN
        ITEMP=MXTRK
        WRITE(NPU,29) ITEMP
29      FORMAT(1X,'ERROR. BOTTOM PROFILE MISSING OR ARRAY OVERFLOW. ',
     C  'MAX IS ',I5, '. CHECK PARAMETER MXTRK.')
        STOP
      ELSE
      ENDIF
C
      R2(L)=TRACK(ITRK(L),1,L)
      Z2(L)=TRACK(ITRK(L),2,L)
C
C     *** ADVANCE TRACK IF NECESSARY SO THAT R1.LE.RA.LT.R2
      IF(RA.GE.R2(L)) GO TO 25
C
C     *** BE SURE THAT R1 LE RA 
      IF(R1(L).GT.RA) THEN
        WRITE(NPU,*)'DEPTH OF BOTTOM AT INITIAL RANGE MISSING,'
	WRITE(NPU,*)'OR PARAMETER MXTRK SHOULD BE INCREASED'
	write(6,*)'L,R1(L),RA ',L,R1(L),RA
        STOP
      ELSE
      ENDIF
C
C     *** COMPUTE SLOPE OF BOTTOM IN RANGE
      THETA(L)=ATAN2(Z2(L)-Z1(L),R2(L)-R1(L))
C
C     *** END OF DO LOOP
31    CONTINUE
C
C     *** READ RANGE OF SVP
      READ(NIU,*,END=33) RSVP
C
C     *** SVP BEYOND START RANGE?
      IF(RSVP.GT.RA) GO TO 33
C
C     *** NO. DETERMINE IF SVP IN RUNSTREAM OR SUPPLIED BY SUBROUTINE USVP3D.
32    READ(NIU,*,END=33) KSVP
      IF(KSVP.EQ.0) CALL SVP3D
      IF(KSVP.NE.0) CALL USVP3D
C
C     *** ERROR IN SVP?
      IF(NSVP.NE.0) GO TO 35
C     *** YES.
33    WRITE(NPU,34) RA
34    FORMAT(1X,'SVP MISSING OR INPUT ERROR. RANGE = ',F8.1,' M.')
      STOP
C
35    CONTINUE
C     *** NO ERROR DETECTED.
C     *** IF C0 NOT SPECIFIED, SET C0 TO AVERAGE SPEED OF FIRST PROFILES
C     *** IN FIRST LAYERS AT INITIAL RANGE
C
      IF(C0.EQ.0.0) THEN
        ZWSVP=0.0
        DO 40 L=1,NSOL
        NWSVP=IXSVP(1,L)
        ZWSVP=ZWSVP+ZSVP(NWSVP,L)
        DO 40 I=2,NWSVP
        C0=C0+(ZSVP(I,L)-ZSVP(I-1,L))*(CSVP(I-1,L)+.5*(CSVP(I,L)-
     C  CSVP(I-1,L)))
40      CONTINUE
        C0=C0/ZWSVP
      ELSE
      ENDIF
C
C     *** COMPUTE REFERENCE WAVE NUMBER
      XK0=2.0*PI*FRQ/C0
C
C     *** COMPUTE ATTENUATION - SACLANT MEMO SM-121 (JENSEN + FERLA)
C     *** MODIFIED AS FOLLOWS:
C     *** IF INPUTTED BETA IS LT 0.0, ALPHA IS COMPUTED IN DB/METER
C     *** AND USED TO COMPUTE BETA IN DB/WAVELENGTH ROUTINE INDX3D.
C
      ALPHA=FRQ*FRQ*(.007+(.155*1.7)/(1.7*1.7+FRQ*FRQ*.000001))*1.0E-09
C
C     *** ADJUST LAYER DEPTHS IN CASE BOTTOM SLOPES AND RA.GT.RSVP.
C     *** ASSUMES LAYERS FOLLOW BOTTOM CONTOUR.
C
      DO 50 L=1,NSOL
      NLYR=NLYRS(L)
      DO 50 ILYR=1,NLYR
      ZLYR(ILYR,L)=ZLYR(ILYR,L)+(RA-RSVP)*TAN(THETA(L))
50    CONTINUE
C
C     *** GET RANGE OF NEXT SVP
C     *** IF END, NO MORE PROFILES FOLLOW.
C
53    READ(NIU,*,END=55) RSVP
C
C     *** TEST FOR 3D INPUT RUNSTREAM - USED DURING DEBUG PHASE.
C     *** WHEN COMPARING 2D SOLUTION WITH AZIMUTH INDEPENDENT NX2D SOLUTIONS.
C     *** SEE NOTE 2 REGARDING -1,-1 IN DESCRIPTION OF INPUT PARAMETERS.
C 
      IF(NDIM.EQ.1.AND.RSVP.LT.0.0)GO TO 53
      GO TO 56
C
C     *** ONLY ONE SVP - SET RSVP LARGE SO SAME SVP USED FOR ENTIRE PROBLEM
55    RSVP=1.0E+38
C
56    CONTINUE
C
C     *** IF STARTING FIELD IS BEYOND NEXT SVP, GO BACK AND GET NEXT SVP
      IF(RSVP.LE.RA) GO TO 32
C
C     *** ABSORBING LAYER REQUESTED?
      IF(ITYPEB.EQ.3) THEN
C       *** YES.
C       *** EXTEND BOTTOM 4/3 MAX DEPTH IF ZA = ZERO
        IF(ZA.EQ.0.0) ZA=4.0*ZBOT/3.0
C       *** ABSORBING LAYER DEEP ENOUGH?
        IF(ZA.LT.ZBOT) THEN
C         *** NO. EXTEND BOTTOM 4/3 MAX DEPTH. 
          WRITE(NPU,57)
57        FORMAT(1X,'ERROR. ZA RESET TO MAX DEPTH OF BOTTOM LAYER.')
          ZA=4.0*ZBOT/3.0
        ELSE
        ENDIF
C
C       *** INSERT PARAMETERS FOR EXTENDED BOTTOM IN APPROPRIATE ARRAYS
        ZMAX=ZA
        DO 59 L=1,NSOL
        NLYR=NLYRS(L)
        ZG=ZLYR(NLYR,L)
        NSVP=IXSVP(NLYR,L)
        IF(NLYR.GT.1)ZG=ZG-ZLYR(NLYR-1,L)
        NLYRS(L)=NLYRS(L)+1
        NLYR=NLYRS(L)
C       *** STORE DEPTH OF ARTIFICIAL ABSORBING LAYER
        ZLYR(NLYR,L)=ZA
C       *** USE BOTTOM DENSITY AND ATTENUATION OF ABOVE LAYER 
        RHO(NLYR,L)=RHO(NLYR-1,L)+ZG*RHOG(NLYR-1,L)
        RHOG(NLYR,L)=0.0
        BETA(NLYR,L)=BETA(NLYR-1,L)+ZG*BETAG(NLYR-1,L)
        BETAG(NLYR,L)=0.0
C       *** USE BOTTOM SPEED OF ABOVE LAYER
        IXSVP(NLYR,L)=NSVP+2
        ZSVP(NSVP+1,L)=ZSVP(NSVP,L)
        CSVP(NSVP+1,L)=CSVP(NSVP,L)
        ZSVP(NSVP+2,L)=ZLYR(NLYR,L)
        CSVP(NSVP+2,L)=CSVP(NSVP+1,L)
59      CONTINUE
      ELSE
C       *** IF BOTTOM NOT EXTENDED AND ZA=0, SET ZA TO MAX DEPTH INPUTTED
C       *** WITH FIRST SVP. ASSUMES DEPTH IS CONSTANT IN AZIMUTH.
        IF(ZA.EQ.0.0) ZA=ZLYR(NLYR,1)
      ENDIF
C
C     *** IF N NOT SPECIFIED, SET N SO THAT DZ .LE. 1/10 WAVELENGTH  
62    IF(N.EQ.0) THEN
        FN=((10*ZA*FRQ/C0)+1)
        N=INT(FN)
      ELSE
      ENDIF
C
      IF(N.GT.MXNZ) THEN
        WRITE(NPU,*)'ERROR. N TOO LARGE.'
        WRITE(NPU,*)'INCREASE PARAMETER MXNZ AND RECOMPILE.'
        STOP
      ELSE
      ENDIF
C
C     *** COMPUTE RECEIVER DEPTH INCREMENT. DZ MAY BE SUCH THAT RECEIVERS DO 
C     *** NOT LIE EXACTLY ON LAYER INTERFACES
      DZ=ZA/N
C
C     ADJUST N SO THAT DEPTH (N+1)*DZ IS BOTTOM
      IF(ITYPEB.EQ.1.OR.ITYPEB.EQ.3)N=N-1  
C
C     *** COMPUTE TOTAL SOLUTION DEPTHS
      NTOT=N*NSOL
C
C     *** IF RANGE STEP NOT SPECIFIED, SET IT EQUAL TO 1/10 WAVELENGTH.
C     *** THIS STEP SIZE IS ARBITRARY.
C
      IF(DR.EQ.0.0) DR=.1*C0/FRQ
C
C     *** GET STARTING FIELD
      CALL SFLD3D
C
C     *** COMPUTE DEPTH WRITE INCREMENT TO NEAREST DZ
      IF(WZ1.LT.DZ)WZ1=DZ
      RIWZ=(WZ1/DZ+.5)
      IWZ1=INT(RIWZ)
C
      IF(WZ2.LT.DZ)WZ2=DZ
      IF(ITYPEB.EQ.1.OR.ITYPEB.EQ.3.AND.WZ2.GT.N*DZ)WZ2=N*DZ
      RIWZ=(WZ2/DZ+.5)
      IWZ2=INT(RIWZ)
C
      IF(WDZ.LT.DZ)WDZ=WZ1
      RIWZ=(WDZ/DZ+.5)
      IWZ=INT(RIWZ)
      WDZ=IWZ*DZ
C
      NZ=(IWZ2-IWZ1)/IWZ+1
C
C     *** COMPUTE DEPTH PRINT INCREMENT TO NEAREST DZ
      RIPZ=(PDZ/DZ+.5)
      IPZ=INT(RIPZ)
      IF(PDZ.GT.0.0.AND.IPZ.EQ.0) IPZ=1
      PDZ=IPZ*DZ
C
C     *** PRINT INPUT PARAMETERS
      CALL PRINTP
C
84    FORMAT(3X,'LAYER ',I3,3X,'DEPTH = ',F8.1,3X,
     C'DENSITY = ',F6.2,3X,'GRADIENT = ',F7.4,/,3X,'BETA = ',F8.3,
     C3X,'GRADIENT = ',F7.4)
93    FORMAT(//1X,'SOUND VELOCITY PROFILE AT RANGE ',F10.1,' METERS',/)
94    FORMAT(1X,I4,3X,F8.1,3X,F8.1)
C
C     *** OUTPUT REQUESTED?
      IF(WDR.NE.0.0) THEN
C       *** YES. WRITE SELECTED PARAMETERS TO OUTPUT FILE.
	CALL ASSIGN(NOU,'FOR3D.OUT')
        WRITE(NOU,994)NDIM,FRQ,ZS,C0,ISF,RA,ZA,N,IHNK,ITYPES,ITYPEB,
     C  ITYPPW,ITYPSW,FLDW,NSEC,NSOL,RMAX,DR,WDR,WZ1,WZ2,WDZ,DZ,
     C  DOUGRA,NDIV,U1,U2,U3,U4,U5,U6,U7,U8,U9,U10,U11,U12
994     FORMAT(1X,I1,3F10.2,I3,2F8.0,I5,5I2,F9.3,/,2I5,F10.1,F5.1,
     C  5F7.1,F10.1,I4,/,(5F15.7))
C
C       *** WRITE STARTING FIELD TO OUTPUT FILE.
        DO 97 J=1,NSOL,NSOLW
        M=(J-1)*N
        IF(NDIM.NE.1)ANG=-FLDW/2.0+((J-1)*FLDW/NSEC)
        WRITE(NOU,995) ANG,NZ,RA,WZ1,WDZ,(U(M+I),I=IWZ1,IWZ2,IWZ)
995     FORMAT(1X,F10.5,I6,F8.0,F8.1,F8.1,/,2(E15.7,E15.7))
97      CONTINUE
      ELSE
C       *** NO OUTPUT REQUESTED.
      ENDIF
C
C     *** INITIALIZE RANGE VARIABLE AT WHICH SOLUTION IS TO BE PRINTED
      XPR=RA+PDR
C
C     *** INITIALIZE RANGE VARIABLE AT WHICH SOLUTION IS TO BE RECORDED
      XWR=RA+WDR
      IF(WDR.EQ.0.0) XWR=RA+RMAX+1.0
C
C     *** SAVE RANGE STEP
      OLDR=DR
C
      N1=1
C
C     ******************************************************************
C     ******************************************************************
C     *** MAIN LOOP STARTS HERE                                      ***
C     *** SOLUTION WILL BE ADVANCED FROM RANGE RA TO RANGE RA+DR     ***
C     ******************************************************************
C     ******************************************************************
C
100   CONTINUE
      DRLAST=DR
C
C     *** COMPUTE INDEX OF REFRACTION TABLE AT PRESENT RANGE
      CALL INDX3D
C
C     *** THIS SECTION OF CODE DETERMINES WHETHER OR NOT TO RESTORE STEP
C     *** SIZE TO THE ORIGINAL DR.
C
C     *** COMMENT NEXT SECTION IF INPUT DR IS TO BE USED
      DO 101 L=1,NSOL
      IF(THETA(L).NE.0.0) GO TO 102
101   CONTINUE
C
C     *** FLAT BOTTOM - RESTORE DR IF IT HAS BEEN CHANGED
      DR=OLDR
C
102   CONTINUE
C
      DO 129 L=1,NSOL
      DRMIN(L)=DR
C     *** DOES SVP CHANGE BEFORE BOTTOM PROFILE?
      IF(RSVP.GE.R2(L)) GO TO 105
C     *** YES. DOES SVP CHANGE BEFORE NEXT SOLUTION RANGE?
      IF(RA+DRMIN(L).LE.RSVP) GO TO 126
C     *** YES. ADJUST DR SO THAT SOLUTION ADVANCES TO RSVP
      DRMIN(L)=RSVP-RA
      GO TO 126
C     *** BOTTOM PROFILE CHANGES BEFORE OR AT SAME RANGE AS SVP
105   CONTINUE
C
C     *** DETERMINE IF BOTTOM DEPTHS ARE TO BE UPDATED
C     *** FIRST PASS - RA WILL BE .LT. R2
C
C     *** UPDATE BOTTOM DEPTHS?
      IF(RA.GE.R2(L)) GO TO 110
C
C     *** NO.
C     *** DETERMINE IF RANGE STEP TOO LARGE .
      IF(RA+DRMIN(L).LE.R2(L)) GO TO 126
C
C     *** RANGE STEP IS TOO LARGE - RESET DR - ADV SOLUTION TO R2
      DRMIN(L)=R2(L)-RA
      GO TO 126
C
C     *** UPDATE BOTTOM DEPTHS
110   CONTINUE
      R1(L)=R2(L)
      Z1(L)=Z2(L)
      ITRK(L)=ITRK(L)+1
      R2(L)=TRACK(ITRK(L),1,L)
      Z2(L)=TRACK(ITRK(L),2,L)
C
C     *** TWO DEPTHS AT SAME RANGE INDICATE VERTICAL DISCONTINUITY.
C     *** ADVANCE TRACK FORWARD.
      IF(R1(L).EQ.R2(L)) GO TO 110
C
C     *** RESTORE DR IN CASE BOTTOM IS FLAT
      DRMIN(L)=OLDR
C
C     *** COMPUTE SLOPE OF BOTTOM
      THETA(L)=ATAN2(Z2(L)-Z1(L),R2(L)-R1(L))
C
C     *** IF BOTTOM IS NOT FLAT, COMPUTE NEW RANGE STEP
      IF(THETA(L).EQ.0) GO TO 112
C
C     *** COMPUTE NEW RANGE STEP. DR MAY BECOME VERY SMALL.
      DRMIN(L)=ABS(DZ*COS(THETA(L))/SIN(THETA(L)))
C
C     *** IF RANGE STEP TOO LARGE, PRINT WARNING MESSAGE. RECOMPUTE DR.
112   IF(RA+DRMIN(L).LE.R2(L)) GO TO 120
C      WRITE(NPU,115)RA,L,DRMIN(L)
115   FORMAT(1X,'AT RANGE ',F7.0,', ISOL ',I3,', RANGE STEP ',F5.1,
     C' TOO LARGE FOR BOTTOM CONTOUR')
C      STOP  !.....................................
C
C     *** ADJUST DR AND CONTINUE.
      DRMIN(L)=R2(L)-RA
C      WRITE(NPU,*)'DR ADJUSTED TO ',DRMIN(L)
C
120   CONTINUE
      IF(RA+DRMIN(L).GT.RSVP) DRMIN(L)=RSVP-RA
C
C     *** PRINT BOTTOM DEPTHS
      IF(IBOT.EQ.0) GO TO 126
      WRITE(NPU,93) RA
      IF(NDIM.NE.1)ANG=-FLDW/2.0+((L-1)*FLDW/NSEC)
      WRITE(NPU,*)' '
      WRITE(NPU,*)' RELATIVE BEARING =  ',ANG,' DEGREES'
      WRITE(NPU,*)' '
      TH=THETA(L)*DEG
      WRITE(NPU,123) R1(L),Z1(L),R2(L),Z2(L),TH
123   FORMAT(//1X,'BOTTOM PROFILE ',//,1X,
     C'R1 = ',F10.1,' M',/,1X,
     C'Z1 = ',F8.1,' M',/,1X,
     C'R2 = ',F10.1,' M',/,1X,
     C'Z2 = ',F8.1,' M',/,1X,
     C'SLOPE = ',F8.1,' DEG')
126   CONTINUE
C
129   CONTINUE
C     *** END OF LOOP
C
C     *** SELECT SMALLEST DR WITHIN FOLLOWING ARBITRARY LIMITS.
C     *** MAXIMUM ALLOWED IS INPUT DR.
C     *** MINIMUM ALLOWED IS 0.5*(DZ*DZ).
      DR=DRMIN(1)
      DO 128 L=2,NSOL
      IF(DR.GT.DRMIN(L))DR=DRMIN(L)
128   CONTINUE
      IF(DR.LT.0.5*(DZ*DZ))THEN
        DR=.5*DZ*DZ
C        WRITE(NPU,*)'AT RANGE ',RA, ' DR RESET TO ',DR
        ELSE
        ENDIF
      IF(DR.GT.OLDR)THEN
        DR=OLDR
C        WRITE(NPU,*)'AT RANGE ',RA, ' DR RESET TO ',DR
        ELSE
        ENDIF
C
      IF(DR.NE.DRLAST)WRITE(NPU,*)'AT RANGE ',RA,' DR WAS CHANGED FROM 
     C',DRLAST,' TO ',DR
C
C     *** COMPUTE MATRIX A AT PRESENT RANGE
      CALL AMIFD3
C
C     *** IF 3D RUN, COMPUTE B MATRIX.
      IF(NDIM.EQ.3) CALL BMIFD3
C 
C     *** ADVANCE RANGE ONE STEP
      RA=RA+DR
      NOLD=N
C
C     *** COMPUTE RIGHT HAND SIDE
      CALL RHS
C
C     *** MARCH SOLUTION 1 DR FORWARD
      CALL TWOSTEP
C
C     *** APPLY ABSORPTION IF ITYPEB = 3
      IF(ITYPEB.EQ.3) THEN
        DO 180 J=1,NSOL
        NLYR=NLYRS(J)
        RMM=(ZLYR(NLYR-1,J)/DZ)
        MM=INT(RMM)
        NA=N-MM
        IF(NA.GT.0) GO TO 175
        IF(NA.GE.-1) GO TO 180
        WRITE(NPU,174) RA
174     FORMAT(1X,'ERR IN THICKNESS OF ABSORBING LAYER AT ',F8.1,' M')
        STOP   
C
175     CONTINUE
        M=(J-1)*N
        DO 179 I=1,NA
        ATT=EXP(-.01*DR*EXP(-((I-NA)/(NA/3.0))**2))
        U(M+MM+I)=U(M+MM+I)*ATT  
        IF(J.GT.1)GO TO 176
        IF(NDIM.EQ.3.AND.ITYPPW.EQ.2)PORTY(MM+I)=PORTY(MM+I)*ATT
176     IF(J.LT.NSOL)GO TO 179
        IF(NDIM.EQ.3.AND.ITYPSW.EQ.2)STBDY(MM+I)=STBDY(MM+I)*ATT
179     CONTINUE
180     CONTINUE
      ELSE
      ENDIF
C
C     *** IF SOLUTION IS TO BE WRITTEN ON DISK,
C     *** WRITE SELECTED PRESSURE FIELD AT RANGE RA 
      IF(XWR.GT.RA) GO TO 260
C
      IF(WZ1.LT.DZ)WZ1=DZ
      RIWZ=(WZ1/DZ+.5)
      IWZ1=INT(RIWZ)
C
      IF(WZ2.LT.DZ)WZ2=DZ
      IF(ITYPEB.EQ.1.OR.ITYPEB.EQ.3.AND.WZ2.GT.N*DZ)WZ2=N*DZ
      RIWZ=(WZ2/DZ+.5)
      IWZ2=INT(RIWZ)
      IF(WDZ.LT.DZ)WDZ=WZ1
C
      RIWZ=(WDZ/DZ+.5)
      IWZ=INT(RIWZ)
      WDZ=IWZ*DZ
C
      NZ=(IWZ2-IWZ1)/IWZ+1
C
      DO 259 J=1,NSOL,NSOLW
      M=(J-1)*N
      IF(NDIM.NE.1)ANG=-FLDW/2.0+((J-1)*FLDW/NSEC)
      WRITE(NOU,995) ANG,NZ,RA,WZ1,WDZ,(U(M+I),I=IWZ1,IWZ2,IWZ)
259   CONTINUE
C
C     *** DETERMINE NEXT RANGE AT WHICH TO WRITE SOLUTION ON DISK
      XWR=XWR+WDR
260   CONTINUE
C
C     *** DETERMINE IF SOLUTION IS TO BE PRINTED
      IF(XPR.GT.RA.OR.IPZ.EQ.0.OR.PDR.EQ.0.0) GO TO 350
C
C     *** PRINT RANGE
      RTEMP=RA/1000.0
C
C     *** COMPUTE AND PRINT PROPAGATION LOSS AT EACH IPZ'TH DEPTH
275   FORMAT(2X,'RECEIVER',4X,'DEPTH(M)',4X,'LOSS(DB)',14X,'U(I)')
      JSOL=NSOL/2
      DO 300 J=1,NSOL,NSOLP
      M=(J-1)*N
      IF(NDIM.NE.1)ANG=-FLDW*PI*(.5*NSEC-(J-1))/(180.0*NSEC)  
      WRITE(NPU,277)J,RA,ANG*DEG
277   FORMAT(/,2X,'SOLUTION',I4,',  RANGE = ',F8.1,' M',
     C',  RELATIVE BEARING = ',F6.1,
     C' DEGREES',/)
      WRITE(NPU,275)
      UA=0
      DO 295 I=IPZ,N,IPZ
      ZI=I*DZ
      UA=U(M+I)
      IF(IHNK.EQ.1)UA=UA*HNKL(XK0*RA)
      PL=CABS(UA)
      IF(PL.LE.0.0) GO TO 295
      PL=-20.0*ALOG10(PL)
      IF(IHNK.EQ.0) PL=PL+10.0*ALOG10(RA)
      GO TO 289
288   PL=999.9
289   CONTINUE
      WRITE(NPU,294) I,ZI,PL,UA
294   FORMAT(2X,I5,(3X,F10.2,3X,F10.3),3X,'(',E12.5,2X,E12.5,' )')
C     *** UNCOMMENT IF EXACT SOLUTION SUBROUTINE AVAILABLE.
C      CALL UEXACT
      WRITE(NPU,*)' '
295   CONTINUE
300   CONTINUE
C
C     *** DETERMINE NEXT RANGE AT WHICH TO PRINT SOLUTION
      XPR=XPR+PDR
C
350   CONTINUE
C
C     *** SWITCH TO DOUGLAS METHOD?
      IF(DOUGRA.NE.0.0.AND.RA.GE.DOUGRA)THEN
C     *** YES.
        OLDZ=DZ
        NOLD=N
        NTOLD=NTOT
        IF(ITYPEB.EQ.1)N=N+1
        IF(ITYPEB.EQ.3)N=N+1
C
C       *** DIVIDE N BY NDIV AND MAKE NECESSARY ADJUSTMENTS.
        N=INT(N/NDIV+.5)
        DZ=ZA/N
        IF(ITYPEB.EQ.1)N=N-1
        IF(ITYPEB.EQ.3)N=N-1
        NTOT=N*NSOL
        RIPZ=(PDZ/DZ+.5)
        IPZ=INT(RIPZ)
C
C       *** ADJUST SOLUTION FIELD.
        DO 360 J=1,NSOL
        MOLD=(J-1)*NOLD
        M=(J-1)*N
        DO 359 I=1,N
        ZI=I*DZ
C       *** AVOID ROUNDOFF PROBLEM.
        RK=ZI/OLDZ+.5
        K=INT(RK)
        U(M+I)=U(MOLD+K)+(ZI-K*OLDZ)*(U(MOLD+K+1)-U(MOLD+K))/OLDZ
359     CONTINUE
360     CONTINUE
C
C       *** ADJUST WALL CONDITIONS.
        DO 365 I=1,N
        ZI=I*DZ
C       AVOID ROUNDOFF PROBLEM.
        RK=ZI/OLDZ+.5
        K=INT(RK)
        PORTY(I)=PORTY(K)
        STBDY(I)=STBDY(K)
365     CONTINUE
C
C       *** SET ALFA FOR DOUGLAS METHOD. 
        ALFA=1.0/12.0
C       *** SET DOUGRA LARGE SO THAT WE DON'T ENTER THIS CODE AGAIN.
        DOUGRA=1.0E+38
C
        WRITE(NPU,*)' '
        WRITE(NPU,*)'AT RANGE ',RA,' METERS. SWITCH FROM CRANK-NICOLSON 
     CTO DOUGLAS METHOD.'
        WRITE(NPU,*)'CRANK-NICOLSON    N ,DZ, NTOT ',NOLD,OLDZ,NTOLD
        WRITE(NPU,*)'DOUGLAS           N ,DZ, NTOT ',N,     DZ,NTOT
CCCC        CALL SECOND(CPT2)
        WRITE(NPU,370)CPT2-CPT1
370     FORMAT(1X,'ELAPSED TIME = ',F12.2,' CRAY XMP 2/8 CPU SECONDS.')
        WRITE(NPU,*)' '
C
C       *** RECOMPUTE DEPTH WRITE INCREMENT TO NEAREST DZ
        IF(WZ1.LT.DZ)WZ1=DZ
        RIWZ=(WZ1/DZ+.5)
        IWZ1=INT(RIWZ)
C
        IF(WZ2.LT.DZ)WZ2=DZ
        RIWZ=(WZ2/DZ+.5)
        IWZ2=INT(RIWZ)
C
        IF(WDZ.LT.DZ)WDZ=WZ1
        RIWZ=(WDZ/DZ+.5)
        IWZ=INT(RIWZ)
        WDZ=IWZ*DZ
C
        NZ=(IWZ2-IWZ1)/IWZ+1
C
C       *** RECOMPUTE DEPTH PRINT INCREMENT TO NEAREST DZ
        RIPZ=(PDZ/DZ+.5)
        IPZ=INT(RIPZ)
        IF(PDZ.GT.0.0.AND.IPZ.EQ.0) IPZ=1
        PDZ=IPZ*DZ
C
      ELSE
      ENDIF
C
C     *** TERMINATE RUN IF SOLUTION AT MAXIMUM RANGE HAS BEEN OBTAINED
      IF(RA.LT.RMAX)THEN
C
C     *** GET ENVIRONMENT FOR NEXT RANGE
C     *** READ NEW SVP PROFILE FLAG
      IF(RA.GE.RSVP) READ(NIU,*,END=33) KSVP
C
C     *** KSVP MAY HAVE BEEN SET IN USER ROUTINE USVP3D.
C     *** IF KSVP NOT 0, SUBROUTINE USVP3D IS CALLED. USER IS
C     *** RESPONSIBLE FOR ADJUSTING DEPTHS OF LAYERS.
      IF(KSVP.EQ.0) GO TO 534
      CALL USVP3D
      GO TO 548
C
C     *** NEW SVP AT ADVANCED RANGE?
534   IF(RA.GE.RSVP) GO TO 545
C
535   CONTINUE
C
C     *** UPDATE DEPTHS OF LAYERS
C     *** ASSUMES LAYERS ARE PARALLEL AND FOLLOW BOTTOM CONTOUR
C     *** IF ITYPEB = 3, BOTTOM OF ARTIFICIAL LAYER REMAINS FLAT.
      DO 537 L=1,NSOL
      DZZ=DR*TAN(THETA(L))
C     *** IS BOTTOM FLAT?
      IF(THETA(L).EQ.0.0) GO TO 537
      NLYR=NLYRS(L)
      MLYR=NLYRS(L)
      IF(ITYPEB.EQ.3) MLYR=NLYRS(L)-1
        DO 536 ILYR=1,MLYR
        ZLYR(ILYR,L)=ZLYR(ILYR,L)+DZZ
        IF(ITYPEB.EQ.3.AND.ZLYR(NLYR,L).LT.ZLYR(MLYR,L))THEN
          WRITE(NPU,*)'ERROR. ARTIFICIAL LAYER NOT DEEP ENOUGH.'
          STOP
        ELSE
        ENDIF
536     CONTINUE
537   CONTINUE
C
C     *** ADJUST DEPTHS OF PROFILES IN SLOPING LAYERS
C     *** ASSUMES SAME SVP IN LAYERS
      DO 540 L=1,NSOL
      DZZ=DR*TAN(THETA(L))
      NLYR=NLYRS(L)
      NWSVP=IXSVP(1,L)
      NSVP=IXSVP(NLYR,L)
      DO 540 I=NWSVP+1,NSVP
      ZSVP(I,L)=ZSVP(I,L)+DZZ
540   CONTINUE
      GO TO 570
C
C     *** GET NEXT SVP FROM INPUT RUNSTREAM.
545   CONTINUE
      CALL SVP3D
C
548   CONTINUE
C
C     *** ERROR DETECTED?
      IF(NSVP.EQ.0) GO TO 33
C
C     *** NO. READ RANGE OF NEXT PROFILE?
      IF(RA.GE.RSVP) THEN
        READ(NIU,*,END=549) RSVP
C       *** CHECK FOR 3D RUNSTREAM      
        IF(NDIM.EQ.1.AND.RSVP.LT.0.0)READ(NIU,*,END=549) RSVP
      ELSE
      ENDIF
      GO TO 550
C
549   CONTINUE
C     *** SET RSVP LARGE SO THAT LAST PROFILE IS USED FOR REMAINDER OF PROBLEM
      RSVP=1.0E+38
C
550   CONTINUE
C     *** ARTIFICIAL ABSORBING LAYER?
      IF(ITYPEB.EQ.3) THEN
C       *** YES. UPDATE DENSITY, ATTEN AND SPEED.
        DO 551 L=1,NSOL
        NLYR=NLYRS(L)
        ZG=ZLYR(NLYR,L)
        NSVP=IXSVP(NLYR,L)
        IF(ZA.LT.ZLYR(NLYR,L)) GO TO 551
        IF(NLYR.GT.1)ZG=ZLYR(NLYR,L)-ZLYR(NLYR-1,L)
        NLYRS(L)=NLYRS(L)+1
        NLYR=NLYRS(L)
        ZLYR(NLYR,L)=ZA
        RHO(NLYR,L)=RHO(NLYR-1,L)+ZG*RHOG(NLYR-1,L)
        RHOG(NLYR,L)=0.0
        BETA(NLYR,L)=BETA(NLYR-1,L)+ZG*BETAG(NLYR-1,L)
        BETAG(NLYR,L)=0.0
        IXSVP(NLYR,L)=NSVP+2
        ZSVP(NSVP+1,L)=ZSVP(NSVP,L)
        CSVP(NSVP+1,L)=CSVP(NSVP,L)
        ZSVP(NSVP+2,L)=ZLYR(NLYR,L)
        CSVP(NSVP+2,L)=CSVP(NSVP+1,L)
551     CONTINUE
      ELSE
      ENDIF
C
C     *** PRINT SVP
      IF(ISVP.NE.0) THEN
        WRITE(NPU,93) RA
        DO 562 L=1,NSOL
        IF(NDIM.NE.1)ANG=-FLDW/2.0+((L-1)*FLDW/NSEC)
        WRITE(NPU,*)' '
        WRITE(NPU,*)' RELATIVE BEARING =  ',ANG,' DEGREES'
        WRITE(NPU,*)' '
        K=0
        NLYR=NLYRS(L)
          DO 561 ILYR=1,NLYR
          WRITE(NPU,84)ILYR,ZLYR(ILYR,L),RHO(ILYR,L),RHOG(ILYR,L),
     C    BETA(ILYR,L),BETAG(ILYR,L)
          WRITE(NPU,*)' '
          J=K+1
          K=IXSVP(ILYR,L)
            DO 560 I=J,K
            WRITE(NPU,94) I,ZSVP(I,L),CSVP(I,L)
560         CONTINUE
          WRITE(NPU,*)' '
561       CONTINUE
562     CONTINUE
      ELSE
      ENDIF
570   CONTINUE
C
      GO TO 100
      ELSE
      ENDIF
C 
C     *** TERMINATE RUN
600   CONTINUE
      IF(WDR.NE.0.0)CALL CLOSE(NOU)
CCC      CALL SECOND(CPT2)
CCC      WRITE(NPU,601)CPT2-CPT1
601   FORMAT(1X,'END OF RUN. CRAY XMP 2/8 CPU TIME = ',F12.2,
     C' SECONDS.')
      STOP
      END
C                                                             **********
C                                                             * AMIFD3 *
C                                                             **********
      SUBROUTINE AMIFD3
C     ******************************************************************
C     ******************************************************************
C     * COMPUTE DIAGONALS FOR 'A' MATRIX                               *
C     ******************************************************************
      INCLUDE 'FOR3D.CMN'
      COMPLEX T1,T3,XN12,XNSUM
C     *** RA HAS BEEN INCREMENTED BY DR IN MAIN PROGRAM
      T1=CMPLX(.25,-.25*XK0*DR)
      T2=1.0/(XK0*XK0*DZ*DZ)
      T3=T1*T2
      DO 100 L=1,NSOL
      M=(L-1)*N
      DO 50 I=1,N
      XN12=T1*((XN1(M+I)-1)/RHO1(M+I)+(XN2(M+I)-1)/RHO2(M+I))
      R12=1.0/RHO1(M+I)+1.0/RHO2(M+I)
      RSUM=RHO1(M+I)+RHO2(M+I)
      XNSUM=T1*(XN1(M+I)+XN2(M+I)-2.0)
***   UPPER DIAGONAL
      AU(M+I)=(T3+ALFA+ALFA*T1*(XN2(M+I)-1))/RHO2(M+I)
      IF(I.EQ.N) AU(M+I)=0.0
***   MAIN DIAGONAL
      AM(M+I)=(2.0+XNSUM)/RSUM-(ALFA+T3)*R12-ALFA*XN12
***   LOWER DIAGONAL
      AL(M+I)=(T3+ALFA+ALFA*T1*(XN1(M+I)-1))/RHO1(M+I)
      IF(I.EQ.1)AL(M+I)=0.0
50    CONTINUE
100   CONTINUE
      RETURN
      END
C                                                             **********
C                                                             * BCON3D *
C                                                             **********
      SUBROUTINE BCON3D
C     *****************************************************************
C     *** BOTTOM BOUNDARY CONDITION SUBROUTINE
C     *****************************************************************
      INCLUDE 'FOR3D.CMN'
      GO TO (100,200,300,400,500),ITYPEB+1
      WRITE(NPU,50)ITYPEB
50    FORMAT(1X,'ERROR - BOTTOM BOUNDARY CONDITION ',I2,
     C' NOT IMPLEMENTED')
      IERR=1
      RETURN
C
100   CONTINUE
C     *** ITYPEB=0
C     *** PRESSURE RELEASE BOTTOM
      DO 150 L=1,NSOL+2
      BOTY(L)=0.0
      BOTX(L)=0.0
150   CONTINUE
      RETURN
C
200   CONTINUE
C     *** ITYPEB=1
C     *** USER BOTTOM BOUNDARY CONDITION - USER WRITES SUBROUTINE UBCON3D
      CALL UBCON3D
      RETURN
C
300   CONTINUE
C     *** ITYPEB=2
      IERR=1
      WRITE(NPU,50)ITYPEB
      RETURN
C
400   CONTINUE
C     *** ITYPEB=3
C     *** ARTIFICIAL ABSORBING BOTTOM - FLAT
      DO 450 L=1,NSOL+2
      BOTY(L)=0.0
      BOTX(L)=0.0
450   CONTINUE
      RETURN
C 
500   CONTINUE
C     *** ITYPEB=4
      IERR=1
      WRITE(NPU,50)ITYPEB
      RETURN
      END
C                                                             **********
C                                                             * BMIFD3 *
C                                                             **********
      SUBROUTINE BMIFD3
C     ******************************************************************
C     * COMPUTE DIAGONALS FOR B MATRIX                                 *
C     ******************************************************************
      INCLUDE 'FOR3D.CMN'
      COMPLEX T1,T2
      T1=-CMPLX(0.0,XK0*DR/(4.0*XK0*XK0*(RA+.5*DR)*(RA+.5*DR)*PHI*PHI))
      DO 100 I=1,N 
      M=(I-1)*NSOL
      DO 50 L=1,NSOL   
      LM1=L-1
      IF(LM1.EQ.0)LM1=1
      LP1=L
      IF(LP1.GT.NSOL)LP1=NSOL
      J=(LM1-1)*N
      RH3=RHO1(J+I)
      J=(LP1-1)*N
      RH4=RHO1(J+I)
      R34=(1.0/RH3+1.0/RH4)
C     *** UPPER DIAGONAL
      BU(M+L)=.5*(rh3+rh4)*(T1)/RH4
      IF(L.EQ.NSOL)BU(M+L)=0.0
C     *** MAIN DIAGONAL
      BM(M+L)=1.0+(-T1)*.5*(rh3+rh4)*R34
C     *** LOWER DIAGONAL
      BL(M+L)=.5*(rh3+rh4)*(T1)/RH3
      IF(L.EQ.1)BL(M+L)=0.0
50    CONTINUE
100   CONTINUE
      RETURN
      END
C                                                             **********
C                                                             *  HNKL  *
C                                                             **********
      COMPLEX FUNCTION HNKL(X)
C     ******************************************************************
C     * HANKEL FUNCTION H0(1) - POLYNOMIAL APPROXIMATION               *
C     * HANDBOOK OF MATH FUNCTIONS - N.B.S. - NOV 1967                 *
C     ******************************************************************
C
      REAL J0
      DATA PI/3.141592654/
C
      IF(X.GT.3.) GO TO 10
C
C     *** (-3.0.LE.X.LE.3.0)
      Y=X*X/9.0
      J0=1.+Y*(-2.2499997+Y*(+1.2656208+Y*(-0.3163866+Y*(+0.0444479
     C     +Y*(-0.0039444+Y*(+0.0002100))))))
C
C     *** (0.0.LT.X.LE.3.0)
      Y0=2.0*LOG(0.5*X)*J0/PI+0.36746691
     C     +Y*(+0.60559366+Y*(-0.74350384+Y*(+0.25300117+Y*(-0.04261214
     C     +Y*(+0.00427916+Y*(-0.00024846))))))
      HNKL=CMPLX(J0,Y0)
      RETURN
C
C     *** (3.0.LE.X.LT.INFINITY)
10    Y=3.0/X
      F0=  0.79788456+Y*(-0.00000077+Y*(-0.00552740+Y*(-0.00009512
     C               +Y*(+0.00137237+Y*(-0.00072805+Y*(+0.00014476))))))
      T0=X-0.78539816+Y*(-0.04166397+Y*(-0.00003954+Y*(+0.00262573
     C               +Y*(-0.00054125+Y*(-0.00029333+Y*(+0.00013558))))))
      HNKL=F0*CEXP(CMPLX(0.0,T0))/SQRT(X)
      RETURN
      END
C                                                             **********
C                                                             * INDX3D *
C                                                             **********
      SUBROUTINE INDX3D
C     ******************************************************************
C     * COMPUTE INDEX OF REFRACTION TABLE
C     ******************************************************************
      INCLUDE 'FOR3D.CMN'
      DIMENSION COEF(4,MXSVP),WORK(MXSVP+4),WORK2(MXSVP+4)
      DO 200 ISOL=1,NSOL
      MM=(ISOL-1)*N
C     *** GET INDICES, DENSITY, AND ATTENUATION FOR FIRST LAYER
      ILYR=1
      L=1
      M=IXSVP(ILYR,ISOL)
      RH1=RHO(ILYR,ISOL)
      RH1G=RHOG(ILYR,ISOL)
      BETA1=BETA(ILYR,ISOL)
      BETA1G=BETAG(ILYR,ISOL)
      ZG1=0.0
      NLYR=NLYRS(ISOL)
      DO 100 I=1,N
C     *** ZI IS RECEIVER DEPTH
      ZI=I*DZ
C     *** IS RECEIVER IN THIS LAYER?
20    IF(ZI.LE.ZLYR(ILYR,ISOL)) GO TO 49
C     *** NO. ALL LAYERS CHECKED?
      IF(ILYR.EQ.NLYR) GO TO 52
      ZG1=ZLYR(ILYR,ISOL)
C     *** NO. SET INDICES, DENSITY, AND ATTENUATION FOR NEXT LAYER.
      ILYR=ILYR+1
      L=M+1
      M=IXSVP(ILYR,ISOL)
      RH1=RHO(ILYR,ISOL)
      RH1G=RHOG(ILYR,ISOL)
      BETA1=BETA(ILYR,ISOL)
      BETA1G=BETAG(ILYR,ISOL)
      GO TO 20
C
C     *** DEPTH ZI IS IN THIS LAYER.
49    CONTINUE
C     *** DETERMINE SOUND SPEED CI AT DEPTH ZI
      DO 50 J=L,M
      IF(ZI.GT.ZSVP(J,ISOL)) GO TO 50
      L=J
C     *** INTERPOLATE
      CI=CSVP(J-1,ISOL)+(CSVP(J,ISOL)-CSVP(J-1,ISOL))*(ZI-ZSVP(J-1,ISOL)
     C)/(ZSVP(J,ISOL)-ZSVP(J-1,ISOL))
      GO TO 60
50    CONTINUE
C
C     *** EXTRAPOLATE
52    CONTINUE
      IF(ZSVP(M,ISOL).NE.ZSVP(M-1,ISOL))GO TO 53
      CI=CSVP(M,ISOL)
      GO TO 54
53    CI=CSVP(M-1,ISOL)+(CSVP(M,ISOL)-CSVP(M-1,ISOL))*
     C(ZI-ZSVP(M-1,ISOL))/(ZSVP(M,ISOL)-ZSVP(M-1,ISOL))
54    CONTINUE
      IF(IEX.EQ.1) GO TO 60
      WRITE(NPU,56) 
56    FORMAT(1X,'WARNING. EXTRAPOLATION OF SVP PERFORMED.')
      IEX=1
60    CONTINUE
      C1=CI
C     *** IS RECEIVER ON OR WITHIN 1 DZ OF INTERFACE?
      IF(ZLYR(ILYR,ISOL)-ZI.GE.DZ) GO TO 65
C     *** YES. IS THIS BOTTOM LAYER?
      IF(ILYR.EQ.NLYR) GO TO 65
C     *** NO. GET PARAMS FOR MEDIUM 2.
      ZG2=ZLYR(ILYR,ISOL)
      RH2=RHO(ILYR+1,ISOL)
      RH2G=RHOG(ILYR+1,ISOL)
      BETA2=BETA(ILYR+1,ISOL)
      BETA2G=BETAG(ILYR+1,ISOL)
      C2=CSVP(IXSVP(ILYR,ISOL)+1,ISOL)
      GO TO 70
65    CONTINUE
C     *** NOT AN INTERFACE.
      C2=C1
      RH2=RH1
      RH2G=RH1G
      BETA2=BETA1
      BETA2G=BETA1G
      ZG2=ZG1
70    CONTINUE
      RHO1(MM+I)=RH1+(ZI-ZG1)*RH1G
      RHO2(MM+I)=RH2+(ZI+DZ-ZG2)*RH2G
      BETA1=BETA1+(ZI-ZG1)*BETA1G
      BETA2=BETA2+(ZI+DZ-ZG2)*BETA2G
      XN=C0/C1
      IF(BETA1.LT.0.0) BETA1=ALPHA*C1/FRQ
      XN1(MM+I)=CMPLX(XN*XN,XN*XN*BETA1/27.287527)
      XN=C0/C2
      IF(BETA2.LT.0.0)BETA2=ALPHA*C2/FRQ
      XN2(MM+I)=CMPLX(XN*XN,XN*XN*BETA2/27.287527)
100   CONTINUE
200   CONTINUE
      RETURN
      END
C                                                             **********
C                                                             * PORT2D *
C                                                             **********
      SUBROUTINE PORT2D
C     ******************************************************************
C     * SOLVE FOR FIELD AT PORT WALL BOUNDARY
C     * FIELD AT ADVANCED RANGE IN PORTX
C     ******************************************************************
      INCLUDE 'FOR3D.CMN'
C     *** COMPUTE BTA(N1) THROUGH BTA(N)
C     *** ARRAY BTA CONTAINS PARTIAL SOLUTION OF SYSTEM OF EQUATIONS
      BTA(1)=AM(1)
      M=2
      DO 20 I=M,N
      BTA(I)=AM(I)-AL(I)*AU(I-1)/BTA(I-1)
20    CONTINUE
      DO 21 I=1,N
      BTA(I)=1.0/BTA(I)
21    CONTINUE
      TA=0.0
      TB=0.0
C     *** STEP 1 - CALL TRIDIAGONAL SOLVER
      CALL TRID3D(AL(1),AM(1),AU(1),PORTX(1),DP(1),BTA(1),N,N,
     CTA,TB)
C     *** SOLUTION IS IN ARRAY PORTX
      RETURN
      END
C                                                             **********
C                                                             * PORT3D *
C                                                             **********
      SUBROUTINE PORT3D
C     ******************************************************************
C     * PORT SIDEWALL BOUNDARY CONDITION
C     ******************************************************************
      INCLUDE 'FOR3D.CMN'
      COMPLEX T1,T3,T4,TAL,TAM,TAU,XN12,XNSUM
C
      GO TO (100,200,300),ITYPPW+1
      IERR=1
      WRITE(NPU,50) ITYPPW
50    FORMAT(1X,'ERROR - PORT SIDEWALL BOUNDARY CONDITION ',I2,
     C' NOT IMPLEMENTED')
      RETURN
C     
100   CONTINUE
C     *** ITYPPW = 0
C     *** SET FIELD AT PORT SIDEWALL TO 0.0
      DO 150 I=1,N                                             
      PORTX(I)=0.0
      PORTY(I)=0.0
150   CONTINUE
      RETURN                                                     
C     
200   CONTINUE
C     *** ITYPPW = 1 ; USER SUPPLIES SUBROUTINE UPORT3D
      CALL UPORT3D
      RETURN
C     
300   CONTINUE
C     *** ITYPPW = 2
C     *** ASSUMPTION IS THAT ENVIRONMENTAL CONDITIONS AT PORT SIDEWALL 
C     *** BOUNDARY ARE EQUAL TO CONDITIONS AT LEFTMOST SOLUTION BOUNDARY.
C     *** SOLUTION AT BOUNDARY AT ADVANCED RANGE IS SOLVED IN 2D.
      DELTA=XK0*DR
      T1=CMPLX(.25,.25*DELTA)
      T2=1.0/(XK0*XK0*DZ*DZ)
      T3=T1*T2
      T4=CMPLX(.25,-.25*DELTA)
      RH1=RHO1(1)
      RH2=RHO2(1)
      R12=1.0/RH1+1.0/RH2
      XN12=T1*((XN1(1)-1)/RH1+(XN2(1)-1)/RH2)
      RSUM=RH1+RH2
      XNSUM=T1*(XN1(1)+XN2(1)-2.0)
      TAU=(T3+ALFA+ALFA*T1*(XN2(1)-1))/RH2
      TAL=(T3+ALFA+ALFA*T1*(XN1(1)-1))/RH1
      TAM=(2.0+XNSUM)/RSUM-(ALFA+T3)*R12-ALFA*XN12
C     *** COMPUTE RIGHT HAND SIDE D
      DP(1)=TAL*SURY(1)+TAM*PORTY(1)+TAU*PORTY(2)-
     C(T4*T2+ALFA+ALFA*T4*(XN1(1)-1))/RH1*SURX(1)
      DO 350 I=2,N-1
      RH1=RHO1(I)
      RH2=RHO2(I)
      R12=1.0/RH1+1.0/RH2
      XN12=T1*((XN1(I)-1)/RH1+(XN2(I)-1)/RH2)
      RSUM=RH1+RH2
      XNSUM=T1*(XN1(I)+XN2(I)-2.0)
      TAU=(T3+ALFA+ALFA*T1*(XN2(I)-1))/RH2
      TAL=(T3+ALFA+ALFA*T1*(XN1(I)-1))/RH1
      TAM=(2.0+XNSUM)/RSUM-(ALFA+T3)*R12-ALFA*XN12
C     *** COMPUTE RIGHT HAND SIDE D
      DP(I)=TAL*PORTY(I-1)+TAM*PORTY(I)+TAU*PORTY(I+1)
350   CONTINUE
      RH1=RHO1(N)
      RH2=RHO2(N)
      R12=1.0/RH1+1.0/RH2
      XN12=T1*((XN1(N)-1)/RH1+(XN2(N)-1)/RH2)
      RSUM=RH1+RH2
      XNSUM=T1*(XN1(N)+XN2(N)-2.0)
      TAU=(T3+ALFA+ALFA*T1*(XN2(N)-1))/RH2
      TAL=(T3+ALFA+ALFA*T1*(XN1(N)-1))/RH1
      TAM=(2.0+XNSUM)/RSUM-(ALFA+T3)*R12-ALFA*XN12
C     *** COMPUTE RIGHT HAND SIDE D
      DP(N)=TAL*PORTY(N-1)+TAM*PORTY(N)+TAU*BOTY(1)-
     C(T4*T2+ALFA+ALFA*T4*(XN1(N)-1))/RH2*BOTX(1)
      RETURN
      END
C                                                             **********
C                                                             * PRINTP *
C                                                             **********
      SUBROUTINE PRINTP
C     *****************************************************************
C     *** PRINT PROBLEM PARAMETERS
C     *****************************************************************
      INCLUDE 'FOR3D.CMN'
      WRITE(NPU,70) TITLE
70    FORMAT(//1X,80A1)
      WRITE(NPU,72)
72    FORMAT(/1X,'NUSC FOR3D MODEL')
      IF(NDIM.EQ.1)THEN
      WRITE(NPU,*)'2D SOLUTION'
      ELSE
      IF(NDIM.EQ.2)THEN
      WRITE(NPU,*)'N X 2D SOLUTION'
      ELSE
      WRITE(NPU,*)'3D SOLUTION'
      ENDIF
      ENDIF
      IF(ISF.EQ.0) WRITE(NPU,73)
73    FORMAT(1X,'GAUSSIAN STARTING FIELD')
      IF(ISF.EQ.1) WRITE(NPU,74)
74    FORMAT(1X,'USER STARTING FIELD')
      IF(ISF.EQ.2) WRITE(NPU,71)
71    FORMAT(1X,'GREENS STARTING FIELD')
      IF(IHNK.NE.0) WRITE(NPU,75) 
75    FORMAT(1X,'STARTING FIELD DIVIDED BY HANKEL FUNCTION')
      IF(ITYPES.EQ.1) WRITE(NPU,76)
76    FORMAT(1X,'USER SURFACE CONDITION')
      IF(ITYPEB.EQ.1) WRITE(NPU,77)
77    FORMAT(1X,'USER BOTTOM CONDITION')
      IF(ITYPPW.EQ.1) WRITE(NPU,78)
      IF(NDIM.EQ.3.AND.ITYPPW.EQ.2)
     C WRITE(NPU,*)'2D SOLUTION AT PORT SIDEWALL BOUNDARY'
78    FORMAT(1X,'USER PORT SIDEWALL BOUNDARY CONDITION')
      IF(ITYPSW.EQ.1) WRITE(NPU,79)
      IF(NDIM.EQ.3.AND.ITYPSW.EQ.2)
     C WRITE(NPU,*)'2D SOLUTION AT STBD SIDEWALL BOUNDARY'
79    FORMAT(1X,'USER STARBOARD SIDEWALL BOUNDARY CONDITION')
      IF(NDIM.NE.1)ANG=FLDW/NSEC
      WRITE(NPU,80) FRQ,ZS,C0,ISF,RA,ZA,N,IHNK,ITYPES,ITYPEB,
     CITYPPW,ITYPSW,FLDW,NSEC,ANG,NSOL,NTOT
80    FORMAT(1X,'FRQ    = ',F8.2,' HZ',/,1X,
     C'ZS     = ',F8.2,' M',/,1X,
     C'C0     = ',F8.2,' M/SEC',/,1X,
     C'ISF    = ',I5,/,1X,
     C'RA     = ',F8.2,' M',/,1X,
     C'ZA     = ',F8.2,' M',/,1X,
     C'N      = ',I5,/,1X,
     C'IHNK   = ',I5,/,1X,
     C'ITYPES = ',I5,/,1X,
     C'ITYPEB = ',I5,/,1X,
     C'ITYPPW = ',I5,/,1X,
     C'ITYPSW = ',I5,/,1X,
     C'FLDW   = ',F8.3,' (WIDTH OF FIELD IN DEGREES)',/,1X,
     C'NSEC   = ',I5,' SECTORS',/,1X,
     C'PHI    = ',F8.3,' DEGREES/SECTOR',/,1X,
     C'NSOL   = ',I5,' SOLUTIONS',/,1X,
     C'NTOT   = ',I5)
      WRITE(NPU,81) DR,DZ,ANG,WDR,WZ1,WZ2,WDZ,WDTH,PDR,PDZ,PDTH,RMAX,
     CDOUGRA,NDIV,U1,U2,U3,U4,U5,U6,U7,U8,U9,U10,U11,U12
81    FORMAT(1X,'DR     = ',F8.2,' M',/,1X,
     C'DZ     = ',F8.2,' M',/,1X,
     C'DTH    = ',F8.2,' DEGREES',/,1X,
     C'WDR    = ',F8.2,' M',/,1X,
     C'WZ1    = ',F8.2,' M',/,1X,
     C'WZ2    = ',F8.2,' M',/,1X,
     C'WDZ    = ',F8.2,' M',/,1X,
     C'WDTH   = ',F8.2,' DEGREES',/,1X,
     C'PDR    = ',F8.2,' M',/,1X,
     C'PDZ    = ',F8.2,' M',/,1X,
     C'PDTH   = ',F8.2,' DEGREES',/,1X,
     C'RMAX   =',F9.1,' M',/,1X,
     C'DOUGRA =',F9.1,' M',/,1X,
     C'NDIV   = ',I5,/,1X,
     C'USER 1 = ',F12.4,/,1X,
     C'USER 2 = ',F12.4,/,1X,
     C'USER 3 = ',F12.4,/,1X,
     C'USER 4 = ',F12.4,/,1X,
     C'USER 5 = ',F12.4,/,1X,
     C'USER 6 = ',F12.4,/,1X,
     C'USER 7 = ',F12.4,/,1X,
     C'USER 8 = ',F12.4,/,1X,
     C'USER 9 = ',F12.4,/,1X,
     C'USER 10= ',F12.4,/,1X,
     C'USER 11= ',F12.4,/,1X,
     C'USER 12= ',F12.4,/,/)
84    FORMAT(3X,'LAYER ',I3,3X,'DEPTH = ',F8.1,3X,
     C'DENSITY = ',F6.2,3X,'GRADIENT = ',F7.4,/,
     C3X'BETA = ',F8.3,3X,'GRADIENT = ',F7.4)
C
C     *** PRINT BOTTOM DEPTHS IF REQUESTED
      IF(IBOT.EQ.0) GO TO 86
      DO 85 L=1,NSOL
      TH=THETA(L)*DEG
      IF(NDIM.NE.1)ANG=-FLDW/2.0+((L-1)*FLDW/NSEC)
      WRITE(NPU,*)' '
      WRITE(NPU,*)'RELATIVE BEARING =  ',ANG,' DEGREES'
      WRITE(NPU,123) R1(L),Z1(L),R2(L),Z2(L),TH
85    CONTINUE
86    IF(ISFLD.EQ.0) GO TO 92
C
C     *** PRINT STARTING FIELD
      WRITE(NPU,87)
87    FORMAT(/,1X,'STARTING FIELD')
      DO 90 J=1,NSOL
      IF(NDIM.NE.1)ANG=-FLDW/2.0+((J-1)*FLDW/NSEC)
      WRITE(NPU,*)' '
      WRITE(NPU,*)'RELATIVE BEARING = ',ANG,' DEGREES'
      WRITE(NPU,*)' '
      M=(J-1)*N
      DO 90 I=1,N
      ZI=I*DZ
      IF(U(M+I).NE.0.0)WRITE(NPU,89) I,ZI,U(M+I)
89    FORMAT(1X,I4,3X,F10.2,3X,'(',E12.5,2X,E12.5,' )')
90    CONTINUE
92    CONTINUE
C
      IF(ISVP.EQ.0) GO TO 110
C     *** PRINT SVP
      WRITE(NPU,93) RA
93    FORMAT(//1X,'SOUND VELOCITY PROFILE AT RANGE ',F10.1,' METERS',/)
      DO 102 L=1,NSOL
      NLYR=NLYRS(L)
      IF(NDIM.NE.1)ANG=-FLDW/2.0+((L-1)*FLDW/NSEC)
      WRITE(NPU,*)' '
      WRITE(NPU,*)'RELATIVE BEARING = ',ANG,' DEGREES'
      WRITE(NPU,*)' '
      K=0
      DO 101 ILYR=1,NLYR
      WRITE(NPU,84)ILYR,ZLYR(ILYR,L),RHO(ILYR,L),RHOG(ILYR,L),
     CBETA(ILYR,L),BETAG(ILYR,L)
      WRITE(NPU,*)' '
      J=K+1
      K=IXSVP(ILYR,L)
      DO 100 I=J,K
      WRITE(NPU,95) I,ZSVP(I,L),CSVP(I,L)
95    FORMAT(1X,I4,3X,F8.1,3X,F8.1)
100   CONTINUE
      WRITE(NPU,*)' '
101   CONTINUE
102   CONTINUE
110   CONTINUE
      RETURN
123   FORMAT(1X,'BOTTOM DEPTHS ',/,1X,
     C'R1    = ',F10.1,' M',/,1X,
     C'Z1    = ',F8.1,' M',/,1X,
     C'R2    = ',F10.1,' M',/,1X,
     C'Z2    = ',F8.1,' M',/,1X,
     C'THETA = ',F8.1,' DEG')
      END
C                                                             **********
C                                                             *  RHS   *
C                                                             **********
      SUBROUTINE RHS
C     *** COMPUTE RIGHT HAND SIDE.
C     *** RA IS NEXT SOLUTION RANGE.
C     *** CAN SAVE TIME BY COMPUTING 2D AND 3D SOLUTIONS IN SEPARATE
C     ***  SUBROUTINES i.e. RHS2 AND RHS3 ETC.
      INCLUDE 'FOR3D.CMN'
      COMPLEX T1,T3,T5,VM,VMM1,VMP1,TBL,TBM,TBU,TBL2,TBM2,TBU2
      COMPLEX TAL,TAM,TAU,XN12,XNSUM
C     *** GET SURFACE CONDITION
      CALL SCON3D
C     *** GET BOTTOM CONDITION
      CALL BCON3D
C     *** GET PORT WALL CONDITION
      IF(NDIM.EQ.3) CALL PORT3D
C     *** GET STARBOARD WALL CONDITION
      IF(NDIM.EQ.3)CALL STBD3D
      T1=CMPLX(.25,.25*XK0*DR)
      T2=1.0/(XK0*XK0*DZ*DZ)
      T3=T1*T2
      IF(NDIM.EQ.1.OR.NDIM.EQ.2.OR.RA.EQ.0.0)THEN
      T5=0.0
      ELSE
      T5=CMPLX(0.0,XK0*DR/(4.0*XK0*XK0*(RA-.5*DR)*(RA-.5*DR)*PHI*PHI))
      ENDIF
      DO 300 L=1,NSOL
      LP1=L+1
      LP2=L+2
      LM1TN=(L-1)*N
      LTN=L*N
      LM2TN=(L-2)*N
      J=(L-2)*N
      IF(J.LT.0)J=0
      K=(L-1)*N
      DO 200 I=1,N
      RH3=RHO1(J+I)
      RH4=RHO1(K+I)
      R34=(1.0/RH3+1.0/RH4)
      TBU=.5*(rh3+rh4)*(T5)/RH4
      TBM=1.0+(-T5)*.5*(rh3+rh4)*R34
      TBL=.5*(rh3+rh4)*(T5)/RH3
      IF(I.LT.N)THEN
      RH5=RHO1(J+I+1)
      RH6=RHO1(K+I+1)
      R56=(1.0/RH5+1.0/RH6)
      TBU2=.5*(rh5+rh6)*(T5)/RH6
      TBM2=1.0+(-T5)*.5*(rh5+rh6)*R56
      TBL2=.5*(rh5+rh6)*(T5)/RH5
      ELSE
      ENDIF
      IF(NDIM.EQ.1.OR.NDIM.EQ.2) THEN
      TBM=1.0
      TBM2=1.0
      TBL=0.0
      TBU=0.0
      TBL2=0.0
      TBU2=0.0
      ELSE
      ENDIF
      IM1=I-1
      RH1=RHO1(LM1TN+I)
      RH2=RHO2(LM1TN+I)
      R12=1.0/RH1+1.0/RH2
      IF(L.GT.1) GO TO 70
      IF(I.GT.1) GO TO 50
C
C     *** I=1; L=1
      VMM1=TBL*SURY(L)+TBM*SURY(LP1)+TBU*SURY(LP2)
      VM  =TBL*PORTY(I)+TBM*U(LM1TN+I)+TBU*U(LTN+I)
      VMP1=TBL2*PORTY(I+1)+TBM2*U(LM1TN+I+1)+TBU2*U(LTN+I+1)
      GO TO 130
50    CONTINUE
      IF(I.EQ.N) GO TO 60
C
C     *** I=2 TO N-1; L=1
      VMM1=VM
      VM  =VMP1
      VMP1=TBL2*PORTY(I+1)+TBM2*U(LM1TN+I+1)+TBU2*U(LTN+I+1)
      GO TO 130
60    CONTINUE
C
C     *** I=N; L=1
      VMM1=VM
      VM  =VMP1
      VMP1=TBL*BOTY(L)+TBM*BOTY(LP1)+TBU*BOTY(LP2)
      GO TO 130
70    CONTINUE
      IF(L.EQ.NSOL) GO TO 100
      IF(I.GT.1) GO TO 80
C
C     *** I=1; L=2 TO NSOL-1
      VMM1=TBL*SURY(L)+TBM*SURY(LP1)+TBU*SURY(LP2)
      VM  =TBL*U(LM2TN+I)+TBM*U(LM1TN+I)+TBU*U(LTN+I)
      VMP1=TBL2*U(LM2TN+I+1)+TBM2*U(LM1TN+I+1)+TBU2*U(LTN+I+1)
      GO TO 130
80    CONTINUE
      IF(I.EQ.N) GO TO 90
C
C     *** I=2 TO N-1; L=2 TO NSOL-1
      VMM1=VM
      VM  =VMP1
      VMP1=TBL2*U(LM2TN+I+1)+TBM2*U(LM1TN+I+1)+TBU2*U(LTN+I+1)
      GO TO 130
90    CONTINUE
C
C     *** I=N; L=2 TO NSOL-1
      VMM1=VM
      VM  =VMP1
      VMP1=TBL*BOTY(L)+TBM*BOTY(LP1)+TBU*BOTY(LP2)
      GO TO 130
100   CONTINUE
      IF(I.GT.1) GO TO 110
C
C     *** I=1; L=NSOL
      VMM1=TBL*SURY(L)+TBM*SURY(LP1)+TBU*SURY(LP2)
      VM  =TBL*U(LM2TN+I)+TBM*U(LM1TN+I)+TBU*STBDY(I)
      VMP1=TBL2*U(LM2TN+I+1)+TBM2*U(LM1TN+I+1)+TBU2*STBDY(I+1)
      GO TO 130
110   CONTINUE
      IF(I.EQ.N) GO TO 120
C
C     *** I=2 TO N-1; L=NSOL
      VMM1=VM
      VM  =VMP1
      VMP1=TBL2*U(LM2TN+I+1)+TBM2*U(LM1TN+I+1)+TBU2*STBDY(I+1)
      GO TO 130
120   CONTINUE
C
C     *** I=N; L=NSOL
      VMM1=VM
      VM  =VMP1
      VMP1=TBL*BOTY(L)+TBM*BOTY(LP1)+TBU*BOTY(LP2)
130   CONTINUE
      XN12=T1*((XN1(LM1TN+I)-1)/RH1+(XN2(LM1TN+I)-1)/RH2)
      RSUM=RH1+RH2
      XNSUM=T1*(XN1(LM1TN+I)+XN2(LM1TN+I)-2.0)
      TAU=(T3+ALFA+ALFA*T1*(XN2(LM1TN+I)-1))/RH2
      TAL=(T3+ALFA+ALFA*T1*(XN1(LM1TN+I)-1))/RH1
      TAM=(2.0+XNSUM)/RSUM-(ALFA+T3)*R12-ALFA*XN12
C     *** COMPUTE RIGHT HAND SIDE D
      D(LM1TN+I)=TAL*VMM1+TAM*VM+TAU*VMP1
200   CONTINUE
300   CONTINUE
      RETURN
      END
C                                                             **********
C                                                             * SCON3D *
C                                                             **********
      SUBROUTINE SCON3D
C     *****************************************************************
C     *** SURFACE BOUNDARY CONDITION SUBROUTINE
C     *****************************************************************
      INCLUDE 'FOR3D.CMN'
      GO TO (100,200,300),ITYPES+1
      WRITE(NPU,50)ITYPES
50    FORMAT(1X,'ERROR - SURFACE BOUNDARY CONDITION ',I2,
     C' NOT IMPLEMENTED')
      IERR=1
      RETURN
C
100   CONTINUE
C     *** ITYPES=0
C     *** PRESSURE RELEASE SURFACE
      DO 150 L=1,NSOL+2
      SURY(L)=0.0
      SURX(L)=0.0
150   CONTINUE
      RETURN
C
200   CONTINUE
C     *** ITYPES=1
C     *** USER SURFACE CONDITION
      CALL USCON3D
      RETURN
C
300   CONTINUE
C     *** ITYPES = 2
      WRITE(NPU,50)ITYPES
      IERR=1
      RETURN
      END
C                                                             **********
C                                                             * SFLD3D *
C                                                             **********
      SUBROUTINE SFLD3D
C     ***************************************************************
C     *** GENERATE 3D STARTING FIELD
C     ***************************************************************
      INCLUDE 'FOR3D.CMN'
      GO TO (100,200,300,400),ISF+1
      WRITE(NPU,50)ISF
50    FORMAT(1X,'ERROR - STARTING FIELD ISF ',I2,' NOT IMPLEMENTED')
      IERR=1
      RETURN
C
100   CONTINUE
C     *** ISF=0
C     *** GAUSSIAN STARTING FIELD.
      GW=C0/(PI*FRQ)
      GA=1.0/SQRT(GW)
      DO 120 I=1,N                                             
      ZM=I*DZ
      PR=GA*(EXP(-((ZM-ZS)/GW)**2)-EXP(-((-ZM-ZS)/GW)**2))
      U(I)=CMPLX(PR,0.0)
      IF(NDIM.EQ.1)GO TO 120
      DO 110 L=2,NSOL
      M=(L-1)*N+I
      U(M)=U(I)
110   CONTINUE
120   CONTINUE
      IF(NDIM.EQ.1)GO TO 900
      IF(ITYPPW.NE.2) GO TO 135
      DO 130 I=1,N
130   PORTY(I)=U(I)
135   IF(ITYPSW.NE.2) GO TO 900
      DO 140 I=1,N
140   STBDY(I)=U(I)
      GO TO 900
C
200   CONTINUE
C     *** ISF=1
C     *** USER STARTING FIELD 
      CALL USFLD3D
      GO TO 900
C
300   CONTINUE
C     *** ISF=2
C     *** GREENS WIDE ANGLE STARTING FIELD
      XK0=2.0*PI*FRQ/C0
      DO 320 I=1,N                                             
      ZM=I*DZ
      PR=SQRT(XK0)*(1.45-0.42*XK0*XK0*(ZM-ZS)**2.0)*
     CEXP(-(XK0*XK0*(ZM-ZS)**2)/3.0512)
      U(I)=CMPLX(PR,0.0)
      IF(NDIM.EQ.1)GO TO 320
      DO 310 L=2,NSOL
      M=(L-1)*N+I
      U(M)=U(I)
310   CONTINUE
320   CONTINUE
      IF(NDIM.EQ.1)GO TO 900
      IF(ITYPPW.NE.2) GO TO 335
      DO 330 I=1,N
330   PORTY(I)=U(I)
335   IF(ITYPSW.NE.2) GO TO 900
      DO 340 I=1,N
340   STBDY(I)=U(I)
      GO TO 900
C
400   CONTINUE
C     *** ISF=3
      WRITE(NPU,50) ISF
      IERR=1
      RETURN
C
900   CONTINUE
C     *** DIVIDE STARTING FIELD BY HANKEL FUNCTION IF REQUESTED BY USER
      IF(IHNK.EQ.0) GO TO 950
      HNK=HNKL(XK0*RA)
      DO 930 J=1,NSOL
      M=(J-1)*N
      DO 930 I=1,N
      U(M+I)=U(M+I)/HNK
930   CONTINUE
      DO 935 I=1,N
935   PORTY(I)=PORTY(I)/HNK
      DO 945 I=1,N
945   STBDY(I)=STBDY(I)/HNK
950   CONTINUE
      RETURN
      END
C                                                             **********
C                                                             * STBD2D *
C                                                             **********
      SUBROUTINE STBD2D
C     ******************************************************************
C     * SOLVE FOR FIELD AT STBD WALL BOUNDARY
C     * FIELD AT ADVANCED RANGE IN STBDX
C     ******************************************************************
      INCLUDE 'FOR3D.CMN'
C     *** COMPUTE BTA(N1) THROUGH BTA(N)
C     *** ARRAY BTA CONTAINS PARTIAL SOLUTION OF SYSTEM OF EQUATIONS
      J=(NSOL-1)*N
      BTA(1)=AM(J+1)
      M=2
      DO 20 I=M,N
      BTA(I)=AM(J+I)-AL(J+I)*AU(J+I-1)/BTA(I-1)
20    CONTINUE
      DO 21 I=1,N
      BTA(I)=1.0/BTA(I)
21    CONTINUE
      TA=0.0
      TB=0.0
C     *** STEP 1 - CALL TRIDIAGONAL SOLVER
      CALL TRID3D(AL(J+1),AM(J+1),AU(J+1),STBDX(1),DS(1),BTA(1),N,N,
     CTA,TB)
C     *** SOLUTION IS IN ARRAY STBDX
      RETURN
      END
C                                                             **********
C                                                             * STBD3D *
C                                                             **********
      SUBROUTINE STBD3D
C     ******************************************************************
C     * STARBOARD SIDEWALL BOUNDARY CONDITION
C     ******************************************************************
      INCLUDE 'FOR3D.CMN'
      COMPLEX T1,T3,T4,TAL,TAU,TAM,XN12,XNSUM
C
      GO TO (100,200,300),ITYPSW+1
      IERR=1
      WRITE(NPU,50) ITYPSW
50    FORMAT(1X,'ERROR - STARBOARD SIDEWALL BOUNDARY CONDITION ',I2,
     C' NOT IMPLEMENTED')
      RETURN
C     
100   CONTINUE
C     *** ITYPSW = 0
C     *** SET FIELD AT STBD SIDEWALL TO 0.0
      DO 150 I=1,N                                             
      STBDX(I)=0.0
      STBDY(I)=0.0
150   CONTINUE
      RETURN                                                     
C     
200   CONTINUE
C     *** ITYPSW = 1 ; USER SUPPLIES SUBROUTINE USTBD3D
      CALL USTBD3D
      RETURN
C     
300   CONTINUE
C     *** ITYPSW = 2
C     *** ASSUMPTION IS THAT ENVIRONMENTAL CONDITIONS AT STBD SIDEWALL 
C     *** BOUNDARY ARE EQUAL TO CONDITIONS AT RIGHTMOST SOLUTION BOUNDARY.
C     *** SOLUTION AT BOUNDARY AT ADVANCED RANGE IS SOLVED IN 2D.
      DELTA=XK0*DR
      T1=CMPLX(.25,.25*DELTA)
      T2=1.0/(XK0*XK0*DZ*DZ)
      T3=T1*T2
      T4=CMPLX(.25,-.25*DELTA)
      M=(NSOL-1)*N
      RH1=RHO1(M+1)
      RH2=RHO2(M+1)
      R12=1.0/RH1+1.0/RH2
      XN12=T1*((XN1(M+1)-1)/RH1+(XN2(M+1)-1)/RH2)
      RSUM=RH1+RH2
      XNSUM=T1*(XN1(M+1)+XN2(M+1)-2.0)
      TAU=(T3+ALFA+ALFA*T1*(XN2(M+1)-1))/RH2
      TAL=(T3+ALFA+ALFA*T1*(XN1(M+1)-1))/RH1
      TAM=(2.0+XNSUM)/RSUM-(ALFA+T3)*R12-ALFA*XN12
C     *** COMPUTE RIGHT HAND SIDE D
      DS(1)=TAL*SURY(NSOL+2)+TAM*STBDY(1)+TAU*STBDY(2)-
     C(T4*T2+ALFA+ALFA*T4*(XN1(M+1)-1))/RH1*SURX(NSOL+2)
      DO 350 I=2,N-1
      RH1=RHO1(M+I)
      RH2=RHO2(M+I)
      R12=1.0/RH1+1.0/RH2
      XN12=T1*((XN1(M+I)-1)/RH1+(XN2(M+I)-1)/RH2)
      RSUM=RH1+RH2
      XNSUM=T1*(XN1(M+I)+XN2(M+I)-2.0)
      TAU=(T3+ALFA+ALFA*T1*(XN2(M+I)-1))/RH2
      TAL=(T3+ALFA+ALFA*T1*(XN1(M+I)-1))/RH1
      TAM=(2.0+XNSUM)/RSUM-(ALFA+T3)*R12-ALFA*XN12
C     *** COMPUTE RIGHT HAND SIDE D
      DS(I)=TAL*STBDY(I-1)+TAM*STBDY(I)+TAU*STBDY(I+1)
350   CONTINUE
      RH1=RHO1(M+N)
      RH2=RHO2(M+N)
      R12=1.0/RH1+1.0/RH2
      XN12=T1*((XN1(M+N)-1)/RH1+(XN2(M+N)-1)/RH2)
      RSUM=RH1+RH2
      XNSUM=T1*(XN1(M+N)+XN2(M+N)-2.0)
      TAU=(T3+ALFA+ALFA*T1*(XN2(M+N)-1))/RH2
      TAL=(T3+ALFA+ALFA*T1*(XN1(M+N)-1))/RH1
      TAM=(2.0+XNSUM)/RSUM-(ALFA+T3)*R12-ALFA*XN12
C     *** COMPUTE RIGHT HAND SIDE D
      DS(N)=TAL*STBDY(N-1)+TAM*STBDY(N)+TAU*BOTY(NSOL+2)-
     C(T4*T2+ALFA+ALFA*T4*(XN1(M+N)-1))/RH2*BOTX(NSOL+2)
      RETURN
      END
C                                                             **********
C                                                             * SVP3D  *
C                                                             **********
      SUBROUTINE SVP3D
C     ******************************************************************
C     *** SOUND VELOCITY PROFILE SUBROUTINE
C     ******************************************************************
C     *** CALLING PROGRAM SUPPLIES: NOTHING
C     *** SVP SUBROUTINE RETURNS:
C         NLYRS - NUMBER OF LAYERS. LAYER 1 IS WATER. OTHERS ARE SEDIMENT
C         ZLYR - ARRAY - DEPTH OF EACH LAYER. FIRST IS DEPTH OF WATER.
C         RHO  - ARRAY - DENSITY OF EACH LAYER. GRAMS/CUBIC CM
C         RHOG - ARRAY - DENSITY GRADIENT.
C         BETA - ARRAY - ATTENUATION IN EACH LAYER. DB/WAVELENGTH
C         BETAG- ARRAY - ATTENUATION GRADIENT.
C         IXSVP- ARRAY - CONTAINS POINTERS. POINTS TO LAST VALUE OF SVP
C                IN CORRESPONDING LAYER. SVP IS STORED IN ARRAYS ZSVP
C                AND CSVP. IXSVP(1,L) POINTS TO LAST SVP POINT IN WATER
C                ALONG SECTOR BOUNDARY L. L=1,NSOL.
C                IXSVP(NLYR,L) POINTS TO LAST SVP POINT IN BOTTOM-MOST LAYER.
C         NSVP - NUMBER OF POINTS IN ZSVP AND CSVP. ZSVP AND CSVP
C                CONTAIN THE PROFILES FOR ALL LAYERS.
C         ZSVP - ARRAY - SVP DEPTHS - METERS
C         CSVP - ARRAY - SOUND SPEED - METERS/SEC
C     ******************************************************************
C
      INCLUDE 'FOR3D.CMN'
C     *** READ NUMBER OF LAYERS
      DO 75 L=1,NSOL
      NSVP=0
      READ(NIU,*,END=100) NLYRS(L)
      NLYR=NLYRS(L)
C
C     *** IF NLYR IS NEGATIVE, REPEAT LAST PROFILE UNTIL NSOL PROFILES
C     *** HAVE BEEN ENTERED.
      IF(NLYR.LT.0) GO TO 200
      LNLYR=NLYR
      LASTL=L
C     *** FIRST LAYER IS WATER. OTHERS ARE SEDIMENT.
      DO 55 I=1,NLYR
C     *** READ DEPTH OF LAYER, DENSITY AND ATTENUATION.
      READ(NIU,*,END=100) ZLYR(I,L),RHO(I,L),RHOG(I,L),BETA(I,L),
     CBETAG(I,L)
C     *** READ PROFILE.
50    READ(NIU,*,END=100) ZV,CV
      NSVP=NSVP+1
      ZSVP(NSVP,L)=ZV
      CSVP(NSVP,L)=CV
      IF(ZV.LT.ZLYR(I,L)) GO TO 50
      IF(ZV.GT.ZLYR(I,L)) GO TO 100
      IXSVP(I,L)=NSVP
55    CONTINUE
75    CONTINUE
      RETURN
C
C     *** ERROR EXIT
100   NSVP=0
      RETURN
C
C     *** REPEAT LAST PROFILE UNTIL NSOL PROFILES ENTERED.
200   CONTINUE
      NLYR=LNLYR
      NSVP=IXSVP(NLYR,LASTL)
      DO 230 L=LASTL+1,NSOL
      NLYRS(L)=NLYR
      DO 210 I=1,NLYR
      ZLYR(I,L)=ZLYR(I,LASTL)
      RHO(I,L)=RHO(I,LASTL)
      RHOG(I,L)=RHOG(I,LASTL)
      BETA(I,L)=BETA(I,LASTL)
      BETAG(I,L)=BETAG(I,LASTL)
      IXSVP(I,L)=IXSVP(I,LASTL)
210   CONTINUE
      DO 220 J=1,NSVP
      ZSVP(J,L)=ZSVP(J,LASTL)
      CSVP(J,L)=CSVP(J,LASTL)
220   CONTINUE
230   CONTINUE
      RETURN
      END
C                                                             **********
C                                                             * TRID3D *
C                                                             **********
      SUBROUTINE TRID3D(XL,XM,XU,U,D,BTA,L,M,TA,TB)
C     ******************************************************************
C     *** SPECIALIZED VERSION OF METHOD PRESENTED ON PG 442 OF CARNAHAN
C     *** ET AL FOR SOLVING A TRIDIAGONAL MATRIX.
C     *** TRID RETURNS THE SOLUTION FIELD IN U
C     *** BTA IS PARTIAL SOLUTION COMPUTED ELSEWHERE
C     *** D CONTAINS R.H.S.
C     *** GAMMA - TEMPORARY STORAGE 
C     *** TA    - TEMPORARY VARIABLE USED IN MANIPULATION OF BOTTOM POINT
C     *** TB    - TEMPORARY VARIABLE USED IN MANIPULATION OF BOTTOM POINT
C     ******************************************************************
C
      INCLUDE 'FOR3D.PAR'
      COMPLEX XL(MXN),XM(MXN),XU(MXN),U(MXN),GAMMA(MXN),D(MXN),
     C        BTA(MXN),TA,TB
C     *** TA AND TB MAY BE USED AT A LATER DATE.
      GAMMA(1)=D(1)*BTA(1)
      DO 10 I=2,L
      GAMMA(I)=(D(I)-XL(I)*GAMMA(I-1))*BTA(I)
10    CONTINUE
C     *** IF L IS LESS THAN M, U(M) IS KNOWN.
      IF(L.EQ.M) U(M)=GAMMA(M)
      DO 70 I=M,2,-1
      U(I-1)=GAMMA(I-1)-XU(I-1)*U(I)*BTA(I-1)
70    CONTINUE
      RETURN
      END
C                                                             **********
C                                                             * TWOSTEP*
C                                                             **********
      SUBROUTINE TWOSTEP
C     ******************************************************************
C     * SOLVE V
C     * SOLVE U
C     * FIELD AT ADVANCED RANGE IN U
C     ******************************************************************
      INCLUDE 'FOR3D.CMN'
      COMPLEX T1,T2,T3,T4,T5,VM,VMM1,VMP1,TBL,TBM,TBU,TAL,TAU
      COMPLEX VBOTX,VBOTY,VSURX,VSURY
      IF(NDIM.EQ.3.AND.ITYPPW.EQ.2) CALL PORT2D
      IF(NDIM.EQ.3.AND.ITYPSW.EQ.2) CALL STBD2D
      T1=CMPLX(.25,.25*XK0*DR)
      T2=CMPLX(.25,-.25*XK0*DR)
      T4=T1/(XK0*XK0*DZ*DZ)
      IF(NDIM.EQ.1.OR.NDIM.EQ.2.OR.RA.EQ.0.0)THEN
      T5=0.0
      ELSE
      T5=-CMPLX(0.0,XK0*DR/(4.0*XK0*XK0*(RA-.5*DR)*(RA-.5*DR)*PHI*PHI))
      ENDIF
      DO 10 L=1,NSOL
      M=(L-1)*N
      J=(L-2)*N
      IF(J.LT.0)J=0
      RH3=RHO1(J+1)
      K=(L-1)*N
      RH4=RHO1(K+1)
      R34=(1.0/RH3+1.0/RH4)
      TBU=.5*(rh3+rh4)*(T5)/RH4
      TBM=1.0+(-T5)*.5*(rh3+rh4)*R34
      TBL=.5*(rh3+rh4)*(T5)/RH3
      IF(NDIM.EQ.1.OR.NDIM.EQ.2)THEN
        TBL=0.0
        TBM=1.0
        TBU=0.0
      ELSE
      ENDIF
      VSURX=TBL*SURX(L)+TBM*SURX(L+1)+TBU*SURX(L+2)
      T3=CMPLX(.25,-.25*XK0*DR)/(XK0*XK0*DZ*DZ)
      TAL=(T3+ALFA+ALFA*T2*(XN1(M+1)-1))/RHO1(M+1)
      D(M+1)=D(M+1)-TAL*VSURX
      RH3=RHO1(J+N)
      RH4=RHO1(K+N)
      R34=(1.0/RH3+1.0/RH4)
      TBU=.5*(rh3+rh4)*(T5)/RH4
      TBM=1.0+(-T5)*.5*(rh3+rh4)*R34
      TBL=.5*(rh3+rh4)*(T5)/RH3
      IF(NDIM.EQ.1.OR.NDIM.EQ.2)THEN
        TBL=0.0
        TBM=1.0
        TBU=0.0
      ELSE
      ENDIF
      VBOTX=TBL*BOTX(L)+TBM*BOTX(L+1)+TBU*BOTX(L+2)
      TAU=(T3+ALFA+ALFA*T2*(XN2(M+N)-1))/RHO2(M+N)
      D(M+N)=D(M+N)-TAU*VBOTX
10    CONTINUE
C     *** COMPUTE BTA(N1) THROUGH BTA(N)
C     *** ARRAY BTA CONTAINS PARTIAL SOLUTION OF SYSTEM OF EQUATIONS
      BTA(1)=AM(1)
      M=2
      DO 20 I=M,NTOT
      BTA(I)=AM(I)-AL(I)*AU(I-1)/BTA(I-1)
20    CONTINUE
      DO 21 I=1,NTOT
      BTA(I)=1.0/BTA(I)
21    CONTINUE
      TA=0.0
      TB=0.0
C     *** STEP 1 - CALL TRIDIAGONAL SOLVER
      CALL TRID3D(AL,AM,AU,U,D,BTA,NTOT,NTOT,TA,TB)
C     *** SOLUTION IS IN ARRAY U
      IF(NDIM.EQ.1.OR.NDIM.EQ.2) RETURN
      J=0
      DO 50 I=1,N
      DO 50 L=1,NSOL
      J=J+1
      M=(L-1)*N+I
      D(J)=U(M)
50    CONTINUE
      J=(NSOL-1)*N
      DO 60 I=1,N
      M=(I-1)*NSOL
      RH3=RHO1(I)
      RH4=RHO1(J+I)
C      TBL=.5*(rh3+rh3)*(T5)/RH3
      TBL=(T5)
      D(M+1)=D(M+1)-PORTX(I)*TBL
C      TBU=.5*(rh4+rh4)*(T5)/RH4
      TBU=(T5)
      D(M+NSOL)=D(M+NSOL)-STBDX(I)*TBU
60    CONTINUE
105   BTA(1)=BM(1)
      M=2
106   DO 110 I=M,NTOT    
      BTA(I)=BM(I)-BL(I)*BU(I-1)/BTA(I-1)
110   CONTINUE
      DO 111 I=1,NTOT
      BTA(I)=1.0/BTA(I)
111   CONTINUE
      TA=0.0
      TB=0.0
C     *** STEP 2 - CALL TRIDIAGONAL SOLVER
      CALL TRID3D(BL,BM,BU,V,D,BTA,NTOT,NTOT,TA,TB)
C     *** SOLUTION IN V BUT MUST BE REORDERED.
C     *** REORDER SOLUTION
      J=0
      DO 150 I=1,N
      IF(ITYPPW.EQ.2)PORTY(I)=PORTX(I)
      IF(ITYPSW.EQ.2)STBDY(I)=STBDX(I)
      DO 150 L=1,NSOL
      J=J+1
      M=(L-1)*N+I
      U(M)=V(J)
150   CONTINUE
C     *** SOLUTION FIELD AT ADVANCED RANGE IS IN ARRAY U.
      RETURN
      END
